/// Quasigroups for the Damm Algorithm,
/// based on http://www.md-software.de/math/DAMM_Quasigruppen.txt
use std::error::Error;
use std::fmt;

pub fn apply<U: Into<usize>>(a: U, b: U, base: Order) -> u8 {
    let a: usize = a.into();
    let b: usize = b.into();
    match base {
        Order::Base4 => BASE4[a][b],
        Order::Base8 => BASE8[a][b],
        Order::Base10 => BASE10[a][b],
        Order::Base12 => BASE12[a][b],
        Order::Base16 => BASE16[a][b],
        Order::Base24 => BASE24[a][b],
        Order::Base26 => BASE26[a][b],
        Order::Base32 => BASE32[a][b],
        Order::Base36 => BASE36[a][b],
        Order::Base40 => BASE40[a][b],
        Order::Base45 => BASE45[a][b],
        Order::Base56 => BASE56[a][b],
        Order::Base58 => BASE58[a][b],
        Order::Base62 => BASE62[a][b],
        Order::Base64 => BASE64[a][b],
    }
}

#[derive(Clone, Copy)]
pub enum Order {
    Base4,
    Base8,
    Base10,
    Base12,
    Base16,
    Base24,
    Base26,
    Base32,
    Base36,
    Base40,
    Base45,
    Base56,
    Base58,
    Base62,
    Base64,
}

impl Order {
    fn alphabet_length(self) -> u8 {
        match self {
            Order::Base4 => 4,
            Order::Base8 => 8,
            Order::Base10 => 10,
            Order::Base12 => 12,
            Order::Base16 => 16,
            Order::Base24 => 24,
            Order::Base26 => 26,
            Order::Base32 => 32,
            Order::Base36 => 36,
            Order::Base40 => 40,
            Order::Base45 => 45,
            Order::Base56 => 56,
            Order::Base58 => 58,
            Order::Base62 => 62,
            Order::Base64 => 64,
        }
    }
}

impl TryFrom<usize> for Order {
    type Error = UnimplementedBaseError;

    fn try_from(value: usize) -> Result<Self, Self::Error> {
        match value {
            4 => Ok(Order::Base4),
            8 => Ok(Order::Base8),
            10 => Ok(Order::Base10),
            12 => Ok(Order::Base12),
            16 => Ok(Order::Base16),
            24 => Ok(Order::Base24),
            26 => Ok(Order::Base26),
            32 => Ok(Order::Base32),
            36 => Ok(Order::Base36),
            40 => Ok(Order::Base40),
            45 => Ok(Order::Base45),
            56 => Ok(Order::Base56),
            58 => Ok(Order::Base58),
            62 => Ok(Order::Base62),
            64 => Ok(Order::Base64),
            v => Err(UnimplementedBaseError(v)),
        }
    }
}

#[derive(Debug)]
pub struct UnimplementedBaseError(usize);

impl fmt::Display for UnimplementedBaseError {
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
        write!(f, "Provided base ({}) is not implemented", self.0)
    }
}

impl Error for UnimplementedBaseError {}

/// Ordnung 4: x*y:=2x+y mit Rechnung in GF(2^2)
const BASE4: [[u8; 4]; 4] = [[0, 2, 3, 1], [2, 0, 1, 3], [3, 1, 0, 2], [1, 3, 2, 0]];

/// Ordnung 8: x*y:=2x+y mit Rechnung in GF(2^3)
const BASE8: [[u8; 8]; 8] = [
    [0, 2, 4, 6, 3, 1, 7, 5],
    [2, 0, 6, 4, 1, 3, 5, 7],
    [4, 6, 0, 2, 7, 5, 3, 1],
    [6, 4, 2, 0, 5, 7, 1, 3],
    [3, 1, 7, 5, 0, 2, 4, 6],
    [1, 3, 5, 7, 2, 0, 6, 4],
    [7, 5, 3, 1, 4, 6, 0, 2],
    [5, 7, 1, 3, 6, 4, 2, 0],
];

/// Ordnung 10: Computer-Suche, erkennt alle phonetischen Fehler
const BASE10: [[u8; 10]; 10] = [
    [0, 3, 1, 7, 5, 9, 8, 6, 4, 2],
    [7, 0, 9, 2, 1, 5, 4, 8, 6, 3],
    [4, 2, 0, 6, 8, 7, 1, 3, 5, 9],
    [1, 7, 5, 0, 9, 8, 3, 4, 2, 6],
    [6, 1, 2, 3, 0, 4, 5, 9, 7, 8],
    [3, 6, 7, 4, 2, 0, 9, 5, 8, 1],
    [5, 8, 6, 9, 7, 2, 0, 1, 3, 4],
    [8, 9, 4, 5, 3, 6, 2, 0, 1, 7],
    [9, 4, 3, 8, 6, 1, 7, 2, 0, 5],
    [2, 5, 8, 1, 4, 3, 6, 7, 9, 0],
];

/// Ordnung 12: (r,x)*(s,y):=(2r+s,-x+y) mit Rechnung in GF(2^2) x Z_3
const BASE12: [[u8; 12]; 12] = [
    [0, 2, 3, 1, 4, 6, 7, 5, 8, 10, 11, 9],
    [2, 0, 1, 3, 6, 4, 5, 7, 10, 8, 9, 11],
    [3, 1, 0, 2, 7, 5, 4, 6, 11, 9, 8, 10],
    [1, 3, 2, 0, 5, 7, 6, 4, 9, 11, 10, 8],
    [8, 10, 11, 9, 0, 2, 3, 1, 4, 6, 7, 5],
    [10, 8, 9, 11, 2, 0, 1, 3, 6, 4, 5, 7],
    [11, 9, 8, 10, 3, 1, 0, 2, 7, 5, 4, 6],
    [9, 11, 10, 8, 1, 3, 2, 0, 5, 7, 6, 4],
    [4, 6, 7, 5, 8, 10, 11, 9, 0, 2, 3, 1],
    [6, 4, 5, 7, 10, 8, 9, 11, 2, 0, 1, 3],
    [7, 5, 4, 6, 11, 9, 8, 10, 3, 1, 0, 2],
    [5, 7, 6, 4, 9, 11, 10, 8, 1, 3, 2, 0],
];

/// Ordnung 16: x*y:=2x+y mit Rechnung in GF(2^4)
const BASE16: [[u8; 16]; 16] = [
    [0, 2, 4, 6, 8, 10, 12, 14, 3, 1, 7, 5, 11, 9, 15, 13],
    [2, 0, 6, 4, 10, 8, 14, 12, 1, 3, 5, 7, 9, 11, 13, 15],
    [4, 6, 0, 2, 12, 14, 8, 10, 7, 5, 3, 1, 15, 13, 11, 9],
    [6, 4, 2, 0, 14, 12, 10, 8, 5, 7, 1, 3, 13, 15, 9, 11],
    [8, 10, 12, 14, 0, 2, 4, 6, 11, 9, 15, 13, 3, 1, 7, 5],
    [10, 8, 14, 12, 2, 0, 6, 4, 9, 11, 13, 15, 1, 3, 5, 7],
    [12, 14, 8, 10, 4, 6, 0, 2, 15, 13, 11, 9, 7, 5, 3, 1],
    [14, 12, 10, 8, 6, 4, 2, 0, 13, 15, 9, 11, 5, 7, 1, 3],
    [3, 1, 7, 5, 11, 9, 15, 13, 0, 2, 4, 6, 8, 10, 12, 14],
    [1, 3, 5, 7, 9, 11, 13, 15, 2, 0, 6, 4, 10, 8, 14, 12],
    [7, 5, 3, 1, 15, 13, 11, 9, 4, 6, 0, 2, 12, 14, 8, 10],
    [5, 7, 1, 3, 13, 15, 9, 11, 6, 4, 2, 0, 14, 12, 10, 8],
    [11, 9, 15, 13, 3, 1, 7, 5, 8, 10, 12, 14, 0, 2, 4, 6],
    [9, 11, 13, 15, 1, 3, 5, 7, 10, 8, 14, 12, 2, 0, 6, 4],
    [15, 13, 11, 9, 7, 5, 3, 1, 12, 14, 8, 10, 4, 6, 0, 2],
    [13, 15, 9, 11, 5, 7, 1, 3, 14, 12, 10, 8, 6, 4, 2, 0],
];

/// Ordnung 24: (r,x)*(s,y):=(2r+s,-x+y) mit Rechnung in GF(2^3) x Z_3
const BASE24: [[u8; 24]; 24] = [
    [
        0, 2, 4, 6, 3, 1, 7, 5, 8, 10, 12, 14, 11, 9, 15, 13, 16, 18, 20, 22, 19, 17, 23, 21,
    ],
    [
        2, 0, 6, 4, 1, 3, 5, 7, 10, 8, 14, 12, 9, 11, 13, 15, 18, 16, 22, 20, 17, 19, 21, 23,
    ],
    [
        4, 6, 0, 2, 7, 5, 3, 1, 12, 14, 8, 10, 15, 13, 11, 9, 20, 22, 16, 18, 23, 21, 19, 17,
    ],
    [
        6, 4, 2, 0, 5, 7, 1, 3, 14, 12, 10, 8, 13, 15, 9, 11, 22, 20, 18, 16, 21, 23, 17, 19,
    ],
    [
        3, 1, 7, 5, 0, 2, 4, 6, 11, 9, 15, 13, 8, 10, 12, 14, 19, 17, 23, 21, 16, 18, 20, 22,
    ],
    [
        1, 3, 5, 7, 2, 0, 6, 4, 9, 11, 13, 15, 10, 8, 14, 12, 17, 19, 21, 23, 18, 16, 22, 20,
    ],
    [
        7, 5, 3, 1, 4, 6, 0, 2, 15, 13, 11, 9, 12, 14, 8, 10, 23, 21, 19, 17, 20, 22, 16, 18,
    ],
    [
        5, 7, 1, 3, 6, 4, 2, 0, 13, 15, 9, 11, 14, 12, 10, 8, 21, 23, 17, 19, 22, 20, 18, 16,
    ],
    [
        16, 18, 20, 22, 19, 17, 23, 21, 0, 2, 4, 6, 3, 1, 7, 5, 8, 10, 12, 14, 11, 9, 15, 13,
    ],
    [
        18, 16, 22, 20, 17, 19, 21, 23, 2, 0, 6, 4, 1, 3, 5, 7, 10, 8, 14, 12, 9, 11, 13, 15,
    ],
    [
        20, 22, 16, 18, 23, 21, 19, 17, 4, 6, 0, 2, 7, 5, 3, 1, 12, 14, 8, 10, 15, 13, 11, 9,
    ],
    [
        22, 20, 18, 16, 21, 23, 17, 19, 6, 4, 2, 0, 5, 7, 1, 3, 14, 12, 10, 8, 13, 15, 9, 11,
    ],
    [
        19, 17, 23, 21, 16, 18, 20, 22, 3, 1, 7, 5, 0, 2, 4, 6, 11, 9, 15, 13, 8, 10, 12, 14,
    ],
    [
        17, 19, 21, 23, 18, 16, 22, 20, 1, 3, 5, 7, 2, 0, 6, 4, 9, 11, 13, 15, 10, 8, 14, 12,
    ],
    [
        23, 21, 19, 17, 20, 22, 16, 18, 7, 5, 3, 1, 4, 6, 0, 2, 15, 13, 11, 9, 12, 14, 8, 10,
    ],
    [
        21, 23, 17, 19, 22, 20, 18, 16, 5, 7, 1, 3, 6, 4, 2, 0, 13, 15, 9, 11, 14, 12, 10, 8,
    ],
    [
        8, 10, 12, 14, 11, 9, 15, 13, 16, 18, 20, 22, 19, 17, 23, 21, 0, 2, 4, 6, 3, 1, 7, 5,
    ],
    [
        10, 8, 14, 12, 9, 11, 13, 15, 18, 16, 22, 20, 17, 19, 21, 23, 2, 0, 6, 4, 1, 3, 5, 7,
    ],
    [
        12, 14, 8, 10, 15, 13, 11, 9, 20, 22, 16, 18, 23, 21, 19, 17, 4, 6, 0, 2, 7, 5, 3, 1,
    ],
    [
        14, 12, 10, 8, 13, 15, 9, 11, 22, 20, 18, 16, 21, 23, 17, 19, 6, 4, 2, 0, 5, 7, 1, 3,
    ],
    [
        11, 9, 15, 13, 8, 10, 12, 14, 19, 17, 23, 21, 16, 18, 20, 22, 3, 1, 7, 5, 0, 2, 4, 6,
    ],
    [
        9, 11, 13, 15, 10, 8, 14, 12, 17, 19, 21, 23, 18, 16, 22, 20, 1, 3, 5, 7, 2, 0, 6, 4,
    ],
    [
        15, 13, 11, 9, 12, 14, 8, 10, 23, 21, 19, 17, 20, 22, 16, 18, 7, 5, 3, 1, 4, 6, 0, 2,
    ],
    [
        13, 15, 9, 11, 14, 12, 10, 8, 21, 23, 17, 19, 22, 20, 18, 16, 5, 7, 1, 3, 6, 4, 2, 0,
    ],
];

/// Ordnung 26: Insertion-Konstruktion 25+1
const BASE26: [[u8; 26]; 26] = [
    [
        0, 2, 3, 4, 5, 1, 7, 8, 9, 10, 6, 12, 13, 14, 15, 11, 17, 18, 19, 20, 16, 22, 23, 24, 25,
        21,
    ],
    [
        2, 0, 16, 9, 22, 15, 5, 19, 7, 25, 13, 17, 10, 23, 11, 3, 8, 21, 14, 1, 20, 24, 12, 4, 18,
        6,
    ],
    [
        3, 11, 0, 17, 10, 23, 14, 1, 20, 8, 21, 4, 18, 6, 24, 12, 16, 9, 22, 15, 2, 7, 25, 13, 5,
        19,
    ],
    [
        4, 24, 12, 0, 18, 6, 22, 15, 2, 16, 9, 13, 5, 19, 7, 25, 3, 17, 10, 23, 11, 20, 8, 21, 14,
        1,
    ],
    [
        5, 7, 25, 13, 0, 19, 10, 23, 11, 3, 17, 21, 14, 1, 20, 8, 12, 4, 18, 6, 24, 2, 16, 9, 22,
        15,
    ],
    [
        1, 20, 8, 21, 14, 0, 18, 6, 24, 12, 4, 9, 22, 15, 2, 16, 25, 13, 5, 19, 7, 11, 3, 17, 10,
        23,
    ],
    [
        7, 9, 22, 15, 2, 16, 0, 25, 13, 5, 19, 23, 11, 3, 17, 10, 14, 1, 20, 8, 21, 4, 18, 6, 24,
        12,
    ],
    [
        8, 17, 10, 23, 11, 3, 20, 0, 21, 14, 1, 6, 24, 12, 4, 18, 22, 15, 2, 16, 9, 13, 5, 19, 7,
        25,
    ],
    [
        9, 4, 18, 6, 24, 12, 2, 16, 0, 22, 15, 19, 7, 25, 13, 5, 10, 23, 11, 3, 17, 21, 14, 1, 20,
        8,
    ],
    [
        10, 13, 5, 19, 7, 25, 11, 3, 17, 0, 23, 1, 20, 8, 21, 14, 18, 6, 24, 12, 4, 9, 22, 15, 2,
        16,
    ],
    [
        6, 21, 14, 1, 20, 8, 24, 12, 4, 18, 0, 15, 2, 16, 9, 22, 5, 19, 7, 25, 13, 17, 10, 23, 11,
        3,
    ],
    [
        12, 23, 11, 3, 17, 10, 21, 14, 1, 20, 8, 0, 4, 18, 6, 24, 2, 16, 9, 22, 15, 19, 7, 25, 13,
        5,
    ],
    [
        13, 6, 24, 12, 4, 18, 9, 22, 15, 2, 16, 25, 0, 5, 19, 7, 11, 3, 17, 10, 23, 1, 20, 8, 21,
        14,
    ],
    [
        14, 19, 7, 25, 13, 5, 17, 10, 23, 11, 3, 8, 21, 0, 1, 20, 24, 12, 4, 18, 6, 15, 2, 16, 9,
        22,
    ],
    [
        15, 1, 20, 8, 21, 14, 4, 18, 6, 24, 12, 16, 9, 22, 0, 2, 7, 25, 13, 5, 19, 23, 11, 3, 17,
        10,
    ],
    [
        11, 15, 2, 16, 9, 22, 13, 5, 19, 7, 25, 3, 17, 10, 23, 0, 20, 8, 21, 14, 1, 6, 24, 12, 4,
        18,
    ],
    [
        17, 12, 4, 18, 6, 24, 15, 2, 16, 9, 22, 5, 19, 7, 25, 13, 0, 10, 23, 11, 3, 8, 21, 14, 1,
        20,
    ],
    [
        18, 25, 13, 5, 19, 7, 23, 11, 3, 17, 10, 14, 1, 20, 8, 21, 4, 0, 6, 24, 12, 16, 9, 22, 15,
        2,
    ],
    [
        19, 8, 21, 14, 1, 20, 6, 24, 12, 4, 18, 22, 15, 2, 16, 9, 13, 5, 0, 7, 25, 3, 17, 10, 23,
        11,
    ],
    [
        20, 16, 9, 22, 15, 2, 19, 7, 25, 13, 5, 10, 23, 11, 3, 17, 21, 14, 1, 0, 8, 12, 4, 18, 6,
        24,
    ],
    [
        16, 3, 17, 10, 23, 11, 1, 20, 8, 21, 14, 18, 6, 24, 12, 4, 9, 22, 15, 2, 0, 25, 13, 5, 19,
        7,
    ],
    [
        22, 5, 19, 7, 25, 13, 3, 17, 10, 23, 11, 20, 8, 21, 14, 1, 6, 24, 12, 4, 18, 0, 15, 2, 16,
        9,
    ],
    [
        23, 14, 1, 20, 8, 21, 12, 4, 18, 6, 24, 2, 16, 9, 22, 15, 19, 7, 25, 13, 5, 10, 0, 11, 3,
        17,
    ],
    [
        24, 22, 15, 2, 16, 9, 25, 13, 5, 19, 7, 11, 3, 17, 10, 23, 1, 20, 8, 21, 14, 18, 6, 0, 12,
        4,
    ],
    [
        25, 10, 23, 11, 3, 17, 8, 21, 14, 1, 20, 24, 12, 4, 18, 6, 15, 2, 16, 9, 22, 5, 19, 7, 0,
        13,
    ],
    [
        21, 18, 6, 24, 12, 4, 16, 9, 22, 15, 2, 7, 25, 13, 5, 19, 23, 11, 3, 17, 10, 14, 1, 20, 8,
        0,
    ],
];

/// Ordnung 32: x*y:=2x+y mit Rechnung in GF(2^5)
const BASE32: [[u8; 32]; 32] = [
    [
        0, 2, 4, 6, 8, 10, 12, 14, 16, 18, 20, 22, 24, 26, 28, 30, 3, 1, 7, 5, 11, 9, 15, 13, 19,
        17, 23, 21, 27, 25, 31, 29,
    ],
    [
        2, 0, 6, 4, 10, 8, 14, 12, 18, 16, 22, 20, 26, 24, 30, 28, 1, 3, 5, 7, 9, 11, 13, 15, 17,
        19, 21, 23, 25, 27, 29, 31,
    ],
    [
        4, 6, 0, 2, 12, 14, 8, 10, 20, 22, 16, 18, 28, 30, 24, 26, 7, 5, 3, 1, 15, 13, 11, 9, 23,
        21, 19, 17, 31, 29, 27, 25,
    ],
    [
        6, 4, 2, 0, 14, 12, 10, 8, 22, 20, 18, 16, 30, 28, 26, 24, 5, 7, 1, 3, 13, 15, 9, 11, 21,
        23, 17, 19, 29, 31, 25, 27,
    ],
    [
        8, 10, 12, 14, 0, 2, 4, 6, 24, 26, 28, 30, 16, 18, 20, 22, 11, 9, 15, 13, 3, 1, 7, 5, 27,
        25, 31, 29, 19, 17, 23, 21,
    ],
    [
        10, 8, 14, 12, 2, 0, 6, 4, 26, 24, 30, 28, 18, 16, 22, 20, 9, 11, 13, 15, 1, 3, 5, 7, 25,
        27, 29, 31, 17, 19, 21, 23,
    ],
    [
        12, 14, 8, 10, 4, 6, 0, 2, 28, 30, 24, 26, 20, 22, 16, 18, 15, 13, 11, 9, 7, 5, 3, 1, 31,
        29, 27, 25, 23, 21, 19, 17,
    ],
    [
        14, 12, 10, 8, 6, 4, 2, 0, 30, 28, 26, 24, 22, 20, 18, 16, 13, 15, 9, 11, 5, 7, 1, 3, 29,
        31, 25, 27, 21, 23, 17, 19,
    ],
    [
        16, 18, 20, 22, 24, 26, 28, 30, 0, 2, 4, 6, 8, 10, 12, 14, 19, 17, 23, 21, 27, 25, 31, 29,
        3, 1, 7, 5, 11, 9, 15, 13,
    ],
    [
        18, 16, 22, 20, 26, 24, 30, 28, 2, 0, 6, 4, 10, 8, 14, 12, 17, 19, 21, 23, 25, 27, 29, 31,
        1, 3, 5, 7, 9, 11, 13, 15,
    ],
    [
        20, 22, 16, 18, 28, 30, 24, 26, 4, 6, 0, 2, 12, 14, 8, 10, 23, 21, 19, 17, 31, 29, 27, 25,
        7, 5, 3, 1, 15, 13, 11, 9,
    ],
    [
        22, 20, 18, 16, 30, 28, 26, 24, 6, 4, 2, 0, 14, 12, 10, 8, 21, 23, 17, 19, 29, 31, 25, 27,
        5, 7, 1, 3, 13, 15, 9, 11,
    ],
    [
        24, 26, 28, 30, 16, 18, 20, 22, 8, 10, 12, 14, 0, 2, 4, 6, 27, 25, 31, 29, 19, 17, 23, 21,
        11, 9, 15, 13, 3, 1, 7, 5,
    ],
    [
        26, 24, 30, 28, 18, 16, 22, 20, 10, 8, 14, 12, 2, 0, 6, 4, 25, 27, 29, 31, 17, 19, 21, 23,
        9, 11, 13, 15, 1, 3, 5, 7,
    ],
    [
        28, 30, 24, 26, 20, 22, 16, 18, 12, 14, 8, 10, 4, 6, 0, 2, 31, 29, 27, 25, 23, 21, 19, 17,
        15, 13, 11, 9, 7, 5, 3, 1,
    ],
    [
        30, 28, 26, 24, 22, 20, 18, 16, 14, 12, 10, 8, 6, 4, 2, 0, 29, 31, 25, 27, 21, 23, 17, 19,
        13, 15, 9, 11, 5, 7, 1, 3,
    ],
    [
        3, 1, 7, 5, 11, 9, 15, 13, 19, 17, 23, 21, 27, 25, 31, 29, 0, 2, 4, 6, 8, 10, 12, 14, 16,
        18, 20, 22, 24, 26, 28, 30,
    ],
    [
        1, 3, 5, 7, 9, 11, 13, 15, 17, 19, 21, 23, 25, 27, 29, 31, 2, 0, 6, 4, 10, 8, 14, 12, 18,
        16, 22, 20, 26, 24, 30, 28,
    ],
    [
        7, 5, 3, 1, 15, 13, 11, 9, 23, 21, 19, 17, 31, 29, 27, 25, 4, 6, 0, 2, 12, 14, 8, 10, 20,
        22, 16, 18, 28, 30, 24, 26,
    ],
    [
        5, 7, 1, 3, 13, 15, 9, 11, 21, 23, 17, 19, 29, 31, 25, 27, 6, 4, 2, 0, 14, 12, 10, 8, 22,
        20, 18, 16, 30, 28, 26, 24,
    ],
    [
        11, 9, 15, 13, 3, 1, 7, 5, 27, 25, 31, 29, 19, 17, 23, 21, 8, 10, 12, 14, 0, 2, 4, 6, 24,
        26, 28, 30, 16, 18, 20, 22,
    ],
    [
        9, 11, 13, 15, 1, 3, 5, 7, 25, 27, 29, 31, 17, 19, 21, 23, 10, 8, 14, 12, 2, 0, 6, 4, 26,
        24, 30, 28, 18, 16, 22, 20,
    ],
    [
        15, 13, 11, 9, 7, 5, 3, 1, 31, 29, 27, 25, 23, 21, 19, 17, 12, 14, 8, 10, 4, 6, 0, 2, 28,
        30, 24, 26, 20, 22, 16, 18,
    ],
    [
        13, 15, 9, 11, 5, 7, 1, 3, 29, 31, 25, 27, 21, 23, 17, 19, 14, 12, 10, 8, 6, 4, 2, 0, 30,
        28, 26, 24, 22, 20, 18, 16,
    ],
    [
        19, 17, 23, 21, 27, 25, 31, 29, 3, 1, 7, 5, 11, 9, 15, 13, 16, 18, 20, 22, 24, 26, 28, 30,
        0, 2, 4, 6, 8, 10, 12, 14,
    ],
    [
        17, 19, 21, 23, 25, 27, 29, 31, 1, 3, 5, 7, 9, 11, 13, 15, 18, 16, 22, 20, 26, 24, 30, 28,
        2, 0, 6, 4, 10, 8, 14, 12,
    ],
    [
        23, 21, 19, 17, 31, 29, 27, 25, 7, 5, 3, 1, 15, 13, 11, 9, 20, 22, 16, 18, 28, 30, 24, 26,
        4, 6, 0, 2, 12, 14, 8, 10,
    ],
    [
        21, 23, 17, 19, 29, 31, 25, 27, 5, 7, 1, 3, 13, 15, 9, 11, 22, 20, 18, 16, 30, 28, 26, 24,
        6, 4, 2, 0, 14, 12, 10, 8,
    ],
    [
        27, 25, 31, 29, 19, 17, 23, 21, 11, 9, 15, 13, 3, 1, 7, 5, 24, 26, 28, 30, 16, 18, 20, 22,
        8, 10, 12, 14, 0, 2, 4, 6,
    ],
    [
        25, 27, 29, 31, 17, 19, 21, 23, 9, 11, 13, 15, 1, 3, 5, 7, 26, 24, 30, 28, 18, 16, 22, 20,
        10, 8, 14, 12, 2, 0, 6, 4,
    ],
    [
        31, 29, 27, 25, 23, 21, 19, 17, 15, 13, 11, 9, 7, 5, 3, 1, 28, 30, 24, 26, 20, 22, 16, 18,
        12, 14, 8, 10, 4, 6, 0, 2,
    ],
    [
        29, 31, 25, 27, 21, 23, 17, 19, 13, 15, 9, 11, 5, 7, 1, 3, 30, 28, 26, 24, 22, 20, 18, 16,
        14, 12, 10, 8, 6, 4, 2, 0,
    ],
];

/// Ordnung 36: (r,x)*(s,y):=(2r+s,-x+y) mit Rechnung in GF(2^2) x Z_9
const BASE36: [[u8; 36]; 36] = [
    [
        0, 2, 3, 1, 4, 6, 7, 5, 8, 10, 11, 9, 12, 14, 15, 13, 16, 18, 19, 17, 20, 22, 23, 21, 24,
        26, 27, 25, 28, 30, 31, 29, 32, 34, 35, 33,
    ],
    [
        2, 0, 1, 3, 6, 4, 5, 7, 10, 8, 9, 11, 14, 12, 13, 15, 18, 16, 17, 19, 22, 20, 21, 23, 26,
        24, 25, 27, 30, 28, 29, 31, 34, 32, 33, 35,
    ],
    [
        3, 1, 0, 2, 7, 5, 4, 6, 11, 9, 8, 10, 15, 13, 12, 14, 19, 17, 16, 18, 23, 21, 20, 22, 27,
        25, 24, 26, 31, 29, 28, 30, 35, 33, 32, 34,
    ],
    [
        1, 3, 2, 0, 5, 7, 6, 4, 9, 11, 10, 8, 13, 15, 14, 12, 17, 19, 18, 16, 21, 23, 22, 20, 25,
        27, 26, 24, 29, 31, 30, 28, 33, 35, 34, 32,
    ],
    [
        32, 34, 35, 33, 0, 2, 3, 1, 4, 6, 7, 5, 8, 10, 11, 9, 12, 14, 15, 13, 16, 18, 19, 17, 20,
        22, 23, 21, 24, 26, 27, 25, 28, 30, 31, 29,
    ],
    [
        34, 32, 33, 35, 2, 0, 1, 3, 6, 4, 5, 7, 10, 8, 9, 11, 14, 12, 13, 15, 18, 16, 17, 19, 22,
        20, 21, 23, 26, 24, 25, 27, 30, 28, 29, 31,
    ],
    [
        35, 33, 32, 34, 3, 1, 0, 2, 7, 5, 4, 6, 11, 9, 8, 10, 15, 13, 12, 14, 19, 17, 16, 18, 23,
        21, 20, 22, 27, 25, 24, 26, 31, 29, 28, 30,
    ],
    [
        33, 35, 34, 32, 1, 3, 2, 0, 5, 7, 6, 4, 9, 11, 10, 8, 13, 15, 14, 12, 17, 19, 18, 16, 21,
        23, 22, 20, 25, 27, 26, 24, 29, 31, 30, 28,
    ],
    [
        28, 30, 31, 29, 32, 34, 35, 33, 0, 2, 3, 1, 4, 6, 7, 5, 8, 10, 11, 9, 12, 14, 15, 13, 16,
        18, 19, 17, 20, 22, 23, 21, 24, 26, 27, 25,
    ],
    [
        30, 28, 29, 31, 34, 32, 33, 35, 2, 0, 1, 3, 6, 4, 5, 7, 10, 8, 9, 11, 14, 12, 13, 15, 18,
        16, 17, 19, 22, 20, 21, 23, 26, 24, 25, 27,
    ],
    [
        31, 29, 28, 30, 35, 33, 32, 34, 3, 1, 0, 2, 7, 5, 4, 6, 11, 9, 8, 10, 15, 13, 12, 14, 19,
        17, 16, 18, 23, 21, 20, 22, 27, 25, 24, 26,
    ],
    [
        29, 31, 30, 28, 33, 35, 34, 32, 1, 3, 2, 0, 5, 7, 6, 4, 9, 11, 10, 8, 13, 15, 14, 12, 17,
        19, 18, 16, 21, 23, 22, 20, 25, 27, 26, 24,
    ],
    [
        24, 26, 27, 25, 28, 30, 31, 29, 32, 34, 35, 33, 0, 2, 3, 1, 4, 6, 7, 5, 8, 10, 11, 9, 12,
        14, 15, 13, 16, 18, 19, 17, 20, 22, 23, 21,
    ],
    [
        26, 24, 25, 27, 30, 28, 29, 31, 34, 32, 33, 35, 2, 0, 1, 3, 6, 4, 5, 7, 10, 8, 9, 11, 14,
        12, 13, 15, 18, 16, 17, 19, 22, 20, 21, 23,
    ],
    [
        27, 25, 24, 26, 31, 29, 28, 30, 35, 33, 32, 34, 3, 1, 0, 2, 7, 5, 4, 6, 11, 9, 8, 10, 15,
        13, 12, 14, 19, 17, 16, 18, 23, 21, 20, 22,
    ],
    [
        25, 27, 26, 24, 29, 31, 30, 28, 33, 35, 34, 32, 1, 3, 2, 0, 5, 7, 6, 4, 9, 11, 10, 8, 13,
        15, 14, 12, 17, 19, 18, 16, 21, 23, 22, 20,
    ],
    [
        20, 22, 23, 21, 24, 26, 27, 25, 28, 30, 31, 29, 32, 34, 35, 33, 0, 2, 3, 1, 4, 6, 7, 5, 8,
        10, 11, 9, 12, 14, 15, 13, 16, 18, 19, 17,
    ],
    [
        22, 20, 21, 23, 26, 24, 25, 27, 30, 28, 29, 31, 34, 32, 33, 35, 2, 0, 1, 3, 6, 4, 5, 7, 10,
        8, 9, 11, 14, 12, 13, 15, 18, 16, 17, 19,
    ],
    [
        23, 21, 20, 22, 27, 25, 24, 26, 31, 29, 28, 30, 35, 33, 32, 34, 3, 1, 0, 2, 7, 5, 4, 6, 11,
        9, 8, 10, 15, 13, 12, 14, 19, 17, 16, 18,
    ],
    [
        21, 23, 22, 20, 25, 27, 26, 24, 29, 31, 30, 28, 33, 35, 34, 32, 1, 3, 2, 0, 5, 7, 6, 4, 9,
        11, 10, 8, 13, 15, 14, 12, 17, 19, 18, 16,
    ],
    [
        16, 18, 19, 17, 20, 22, 23, 21, 24, 26, 27, 25, 28, 30, 31, 29, 32, 34, 35, 33, 0, 2, 3, 1,
        4, 6, 7, 5, 8, 10, 11, 9, 12, 14, 15, 13,
    ],
    [
        18, 16, 17, 19, 22, 20, 21, 23, 26, 24, 25, 27, 30, 28, 29, 31, 34, 32, 33, 35, 2, 0, 1, 3,
        6, 4, 5, 7, 10, 8, 9, 11, 14, 12, 13, 15,
    ],
    [
        19, 17, 16, 18, 23, 21, 20, 22, 27, 25, 24, 26, 31, 29, 28, 30, 35, 33, 32, 34, 3, 1, 0, 2,
        7, 5, 4, 6, 11, 9, 8, 10, 15, 13, 12, 14,
    ],
    [
        17, 19, 18, 16, 21, 23, 22, 20, 25, 27, 26, 24, 29, 31, 30, 28, 33, 35, 34, 32, 1, 3, 2, 0,
        5, 7, 6, 4, 9, 11, 10, 8, 13, 15, 14, 12,
    ],
    [
        12, 14, 15, 13, 16, 18, 19, 17, 20, 22, 23, 21, 24, 26, 27, 25, 28, 30, 31, 29, 32, 34, 35,
        33, 0, 2, 3, 1, 4, 6, 7, 5, 8, 10, 11, 9,
    ],
    [
        14, 12, 13, 15, 18, 16, 17, 19, 22, 20, 21, 23, 26, 24, 25, 27, 30, 28, 29, 31, 34, 32, 33,
        35, 2, 0, 1, 3, 6, 4, 5, 7, 10, 8, 9, 11,
    ],
    [
        15, 13, 12, 14, 19, 17, 16, 18, 23, 21, 20, 22, 27, 25, 24, 26, 31, 29, 28, 30, 35, 33, 32,
        34, 3, 1, 0, 2, 7, 5, 4, 6, 11, 9, 8, 10,
    ],
    [
        13, 15, 14, 12, 17, 19, 18, 16, 21, 23, 22, 20, 25, 27, 26, 24, 29, 31, 30, 28, 33, 35, 34,
        32, 1, 3, 2, 0, 5, 7, 6, 4, 9, 11, 10, 8,
    ],
    [
        8, 10, 11, 9, 12, 14, 15, 13, 16, 18, 19, 17, 20, 22, 23, 21, 24, 26, 27, 25, 28, 30, 31,
        29, 32, 34, 35, 33, 0, 2, 3, 1, 4, 6, 7, 5,
    ],
    [
        10, 8, 9, 11, 14, 12, 13, 15, 18, 16, 17, 19, 22, 20, 21, 23, 26, 24, 25, 27, 30, 28, 29,
        31, 34, 32, 33, 35, 2, 0, 1, 3, 6, 4, 5, 7,
    ],
    [
        11, 9, 8, 10, 15, 13, 12, 14, 19, 17, 16, 18, 23, 21, 20, 22, 27, 25, 24, 26, 31, 29, 28,
        30, 35, 33, 32, 34, 3, 1, 0, 2, 7, 5, 4, 6,
    ],
    [
        9, 11, 10, 8, 13, 15, 14, 12, 17, 19, 18, 16, 21, 23, 22, 20, 25, 27, 26, 24, 29, 31, 30,
        28, 33, 35, 34, 32, 1, 3, 2, 0, 5, 7, 6, 4,
    ],
    [
        4, 6, 7, 5, 8, 10, 11, 9, 12, 14, 15, 13, 16, 18, 19, 17, 20, 22, 23, 21, 24, 26, 27, 25,
        28, 30, 31, 29, 32, 34, 35, 33, 0, 2, 3, 1,
    ],
    [
        6, 4, 5, 7, 10, 8, 9, 11, 14, 12, 13, 15, 18, 16, 17, 19, 22, 20, 21, 23, 26, 24, 25, 27,
        30, 28, 29, 31, 34, 32, 33, 35, 2, 0, 1, 3,
    ],
    [
        7, 5, 4, 6, 11, 9, 8, 10, 15, 13, 12, 14, 19, 17, 16, 18, 23, 21, 20, 22, 27, 25, 24, 26,
        31, 29, 28, 30, 35, 33, 32, 34, 3, 1, 0, 2,
    ],
    [
        5, 7, 6, 4, 9, 11, 10, 8, 13, 15, 14, 12, 17, 19, 18, 16, 21, 23, 22, 20, 25, 27, 26, 24,
        29, 31, 30, 28, 33, 35, 34, 32, 1, 3, 2, 0,
    ],
];

/// Ordnung 40: (r,x)*(s,y):=(2r+s,-x+y) mit Rechnung in GF(2^3) x Z_5
const BASE40: [[u8; 40]; 40] = [
    [
        0, 2, 4, 6, 3, 1, 7, 5, 8, 10, 12, 14, 11, 9, 15, 13, 16, 18, 20, 22, 19, 17, 23, 21, 24,
        26, 28, 30, 27, 25, 31, 29, 32, 34, 36, 38, 35, 33, 39, 37,
    ],
    [
        2, 0, 6, 4, 1, 3, 5, 7, 10, 8, 14, 12, 9, 11, 13, 15, 18, 16, 22, 20, 17, 19, 21, 23, 26,
        24, 30, 28, 25, 27, 29, 31, 34, 32, 38, 36, 33, 35, 37, 39,
    ],
    [
        4, 6, 0, 2, 7, 5, 3, 1, 12, 14, 8, 10, 15, 13, 11, 9, 20, 22, 16, 18, 23, 21, 19, 17, 28,
        30, 24, 26, 31, 29, 27, 25, 36, 38, 32, 34, 39, 37, 35, 33,
    ],
    [
        6, 4, 2, 0, 5, 7, 1, 3, 14, 12, 10, 8, 13, 15, 9, 11, 22, 20, 18, 16, 21, 23, 17, 19, 30,
        28, 26, 24, 29, 31, 25, 27, 38, 36, 34, 32, 37, 39, 33, 35,
    ],
    [
        3, 1, 7, 5, 0, 2, 4, 6, 11, 9, 15, 13, 8, 10, 12, 14, 19, 17, 23, 21, 16, 18, 20, 22, 27,
        25, 31, 29, 24, 26, 28, 30, 35, 33, 39, 37, 32, 34, 36, 38,
    ],
    [
        1, 3, 5, 7, 2, 0, 6, 4, 9, 11, 13, 15, 10, 8, 14, 12, 17, 19, 21, 23, 18, 16, 22, 20, 25,
        27, 29, 31, 26, 24, 30, 28, 33, 35, 37, 39, 34, 32, 38, 36,
    ],
    [
        7, 5, 3, 1, 4, 6, 0, 2, 15, 13, 11, 9, 12, 14, 8, 10, 23, 21, 19, 17, 20, 22, 16, 18, 31,
        29, 27, 25, 28, 30, 24, 26, 39, 37, 35, 33, 36, 38, 32, 34,
    ],
    [
        5, 7, 1, 3, 6, 4, 2, 0, 13, 15, 9, 11, 14, 12, 10, 8, 21, 23, 17, 19, 22, 20, 18, 16, 29,
        31, 25, 27, 30, 28, 26, 24, 37, 39, 33, 35, 38, 36, 34, 32,
    ],
    [
        32, 34, 36, 38, 35, 33, 39, 37, 0, 2, 4, 6, 3, 1, 7, 5, 8, 10, 12, 14, 11, 9, 15, 13, 16,
        18, 20, 22, 19, 17, 23, 21, 24, 26, 28, 30, 27, 25, 31, 29,
    ],
    [
        34, 32, 38, 36, 33, 35, 37, 39, 2, 0, 6, 4, 1, 3, 5, 7, 10, 8, 14, 12, 9, 11, 13, 15, 18,
        16, 22, 20, 17, 19, 21, 23, 26, 24, 30, 28, 25, 27, 29, 31,
    ],
    [
        36, 38, 32, 34, 39, 37, 35, 33, 4, 6, 0, 2, 7, 5, 3, 1, 12, 14, 8, 10, 15, 13, 11, 9, 20,
        22, 16, 18, 23, 21, 19, 17, 28, 30, 24, 26, 31, 29, 27, 25,
    ],
    [
        38, 36, 34, 32, 37, 39, 33, 35, 6, 4, 2, 0, 5, 7, 1, 3, 14, 12, 10, 8, 13, 15, 9, 11, 22,
        20, 18, 16, 21, 23, 17, 19, 30, 28, 26, 24, 29, 31, 25, 27,
    ],
    [
        35, 33, 39, 37, 32, 34, 36, 38, 3, 1, 7, 5, 0, 2, 4, 6, 11, 9, 15, 13, 8, 10, 12, 14, 19,
        17, 23, 21, 16, 18, 20, 22, 27, 25, 31, 29, 24, 26, 28, 30,
    ],
    [
        33, 35, 37, 39, 34, 32, 38, 36, 1, 3, 5, 7, 2, 0, 6, 4, 9, 11, 13, 15, 10, 8, 14, 12, 17,
        19, 21, 23, 18, 16, 22, 20, 25, 27, 29, 31, 26, 24, 30, 28,
    ],
    [
        39, 37, 35, 33, 36, 38, 32, 34, 7, 5, 3, 1, 4, 6, 0, 2, 15, 13, 11, 9, 12, 14, 8, 10, 23,
        21, 19, 17, 20, 22, 16, 18, 31, 29, 27, 25, 28, 30, 24, 26,
    ],
    [
        37, 39, 33, 35, 38, 36, 34, 32, 5, 7, 1, 3, 6, 4, 2, 0, 13, 15, 9, 11, 14, 12, 10, 8, 21,
        23, 17, 19, 22, 20, 18, 16, 29, 31, 25, 27, 30, 28, 26, 24,
    ],
    [
        24, 26, 28, 30, 27, 25, 31, 29, 32, 34, 36, 38, 35, 33, 39, 37, 0, 2, 4, 6, 3, 1, 7, 5, 8,
        10, 12, 14, 11, 9, 15, 13, 16, 18, 20, 22, 19, 17, 23, 21,
    ],
    [
        26, 24, 30, 28, 25, 27, 29, 31, 34, 32, 38, 36, 33, 35, 37, 39, 2, 0, 6, 4, 1, 3, 5, 7, 10,
        8, 14, 12, 9, 11, 13, 15, 18, 16, 22, 20, 17, 19, 21, 23,
    ],
    [
        28, 30, 24, 26, 31, 29, 27, 25, 36, 38, 32, 34, 39, 37, 35, 33, 4, 6, 0, 2, 7, 5, 3, 1, 12,
        14, 8, 10, 15, 13, 11, 9, 20, 22, 16, 18, 23, 21, 19, 17,
    ],
    [
        30, 28, 26, 24, 29, 31, 25, 27, 38, 36, 34, 32, 37, 39, 33, 35, 6, 4, 2, 0, 5, 7, 1, 3, 14,
        12, 10, 8, 13, 15, 9, 11, 22, 20, 18, 16, 21, 23, 17, 19,
    ],
    [
        27, 25, 31, 29, 24, 26, 28, 30, 35, 33, 39, 37, 32, 34, 36, 38, 3, 1, 7, 5, 0, 2, 4, 6, 11,
        9, 15, 13, 8, 10, 12, 14, 19, 17, 23, 21, 16, 18, 20, 22,
    ],
    [
        25, 27, 29, 31, 26, 24, 30, 28, 33, 35, 37, 39, 34, 32, 38, 36, 1, 3, 5, 7, 2, 0, 6, 4, 9,
        11, 13, 15, 10, 8, 14, 12, 17, 19, 21, 23, 18, 16, 22, 20,
    ],
    [
        31, 29, 27, 25, 28, 30, 24, 26, 39, 37, 35, 33, 36, 38, 32, 34, 7, 5, 3, 1, 4, 6, 0, 2, 15,
        13, 11, 9, 12, 14, 8, 10, 23, 21, 19, 17, 20, 22, 16, 18,
    ],
    [
        29, 31, 25, 27, 30, 28, 26, 24, 37, 39, 33, 35, 38, 36, 34, 32, 5, 7, 1, 3, 6, 4, 2, 0, 13,
        15, 9, 11, 14, 12, 10, 8, 21, 23, 17, 19, 22, 20, 18, 16,
    ],
    [
        16, 18, 20, 22, 19, 17, 23, 21, 24, 26, 28, 30, 27, 25, 31, 29, 32, 34, 36, 38, 35, 33, 39,
        37, 0, 2, 4, 6, 3, 1, 7, 5, 8, 10, 12, 14, 11, 9, 15, 13,
    ],
    [
        18, 16, 22, 20, 17, 19, 21, 23, 26, 24, 30, 28, 25, 27, 29, 31, 34, 32, 38, 36, 33, 35, 37,
        39, 2, 0, 6, 4, 1, 3, 5, 7, 10, 8, 14, 12, 9, 11, 13, 15,
    ],
    [
        20, 22, 16, 18, 23, 21, 19, 17, 28, 30, 24, 26, 31, 29, 27, 25, 36, 38, 32, 34, 39, 37, 35,
        33, 4, 6, 0, 2, 7, 5, 3, 1, 12, 14, 8, 10, 15, 13, 11, 9,
    ],
    [
        22, 20, 18, 16, 21, 23, 17, 19, 30, 28, 26, 24, 29, 31, 25, 27, 38, 36, 34, 32, 37, 39, 33,
        35, 6, 4, 2, 0, 5, 7, 1, 3, 14, 12, 10, 8, 13, 15, 9, 11,
    ],
    [
        19, 17, 23, 21, 16, 18, 20, 22, 27, 25, 31, 29, 24, 26, 28, 30, 35, 33, 39, 37, 32, 34, 36,
        38, 3, 1, 7, 5, 0, 2, 4, 6, 11, 9, 15, 13, 8, 10, 12, 14,
    ],
    [
        17, 19, 21, 23, 18, 16, 22, 20, 25, 27, 29, 31, 26, 24, 30, 28, 33, 35, 37, 39, 34, 32, 38,
        36, 1, 3, 5, 7, 2, 0, 6, 4, 9, 11, 13, 15, 10, 8, 14, 12,
    ],
    [
        23, 21, 19, 17, 20, 22, 16, 18, 31, 29, 27, 25, 28, 30, 24, 26, 39, 37, 35, 33, 36, 38, 32,
        34, 7, 5, 3, 1, 4, 6, 0, 2, 15, 13, 11, 9, 12, 14, 8, 10,
    ],
    [
        21, 23, 17, 19, 22, 20, 18, 16, 29, 31, 25, 27, 30, 28, 26, 24, 37, 39, 33, 35, 38, 36, 34,
        32, 5, 7, 1, 3, 6, 4, 2, 0, 13, 15, 9, 11, 14, 12, 10, 8,
    ],
    [
        8, 10, 12, 14, 11, 9, 15, 13, 16, 18, 20, 22, 19, 17, 23, 21, 24, 26, 28, 30, 27, 25, 31,
        29, 32, 34, 36, 38, 35, 33, 39, 37, 0, 2, 4, 6, 3, 1, 7, 5,
    ],
    [
        10, 8, 14, 12, 9, 11, 13, 15, 18, 16, 22, 20, 17, 19, 21, 23, 26, 24, 30, 28, 25, 27, 29,
        31, 34, 32, 38, 36, 33, 35, 37, 39, 2, 0, 6, 4, 1, 3, 5, 7,
    ],
    [
        12, 14, 8, 10, 15, 13, 11, 9, 20, 22, 16, 18, 23, 21, 19, 17, 28, 30, 24, 26, 31, 29, 27,
        25, 36, 38, 32, 34, 39, 37, 35, 33, 4, 6, 0, 2, 7, 5, 3, 1,
    ],
    [
        14, 12, 10, 8, 13, 15, 9, 11, 22, 20, 18, 16, 21, 23, 17, 19, 30, 28, 26, 24, 29, 31, 25,
        27, 38, 36, 34, 32, 37, 39, 33, 35, 6, 4, 2, 0, 5, 7, 1, 3,
    ],
    [
        11, 9, 15, 13, 8, 10, 12, 14, 19, 17, 23, 21, 16, 18, 20, 22, 27, 25, 31, 29, 24, 26, 28,
        30, 35, 33, 39, 37, 32, 34, 36, 38, 3, 1, 7, 5, 0, 2, 4, 6,
    ],
    [
        9, 11, 13, 15, 10, 8, 14, 12, 17, 19, 21, 23, 18, 16, 22, 20, 25, 27, 29, 31, 26, 24, 30,
        28, 33, 35, 37, 39, 34, 32, 38, 36, 1, 3, 5, 7, 2, 0, 6, 4,
    ],
    [
        15, 13, 11, 9, 12, 14, 8, 10, 23, 21, 19, 17, 20, 22, 16, 18, 31, 29, 27, 25, 28, 30, 24,
        26, 39, 37, 35, 33, 36, 38, 32, 34, 7, 5, 3, 1, 4, 6, 0, 2,
    ],
    [
        13, 15, 9, 11, 14, 12, 10, 8, 21, 23, 17, 19, 22, 20, 18, 16, 29, 31, 25, 27, 30, 28, 26,
        24, 37, 39, 33, 35, 38, 36, 34, 32, 5, 7, 1, 3, 6, 4, 2, 0,
    ],
];

/// Ordnung 45: x*y:=-x+y mit Rechnung in Z_45
const BASE45: [[u8; 45]; 45] = [
    [
        0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24,
        25, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35, 36, 37, 38, 39, 40, 41, 42, 43, 44,
    ],
    [
        44, 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23,
        24, 25, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35, 36, 37, 38, 39, 40, 41, 42, 43,
    ],
    [
        43, 44, 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22,
        23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35, 36, 37, 38, 39, 40, 41, 42,
    ],
    [
        42, 43, 44, 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21,
        22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35, 36, 37, 38, 39, 40, 41,
    ],
    [
        41, 42, 43, 44, 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
        21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35, 36, 37, 38, 39, 40,
    ],
    [
        40, 41, 42, 43, 44, 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19,
        20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35, 36, 37, 38, 39,
    ],
    [
        39, 40, 41, 42, 43, 44, 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18,
        19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35, 36, 37, 38,
    ],
    [
        38, 39, 40, 41, 42, 43, 44, 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17,
        18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35, 36, 37,
    ],
    [
        37, 38, 39, 40, 41, 42, 43, 44, 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16,
        17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35, 36,
    ],
    [
        36, 37, 38, 39, 40, 41, 42, 43, 44, 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15,
        16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35,
    ],
    [
        35, 36, 37, 38, 39, 40, 41, 42, 43, 44, 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14,
        15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 33, 34,
    ],
    [
        34, 35, 36, 37, 38, 39, 40, 41, 42, 43, 44, 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13,
        14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 33,
    ],
    [
        33, 34, 35, 36, 37, 38, 39, 40, 41, 42, 43, 44, 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12,
        13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32,
    ],
    [
        32, 33, 34, 35, 36, 37, 38, 39, 40, 41, 42, 43, 44, 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11,
        12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31,
    ],
    [
        31, 32, 33, 34, 35, 36, 37, 38, 39, 40, 41, 42, 43, 44, 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10,
        11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
    ],
    [
        30, 31, 32, 33, 34, 35, 36, 37, 38, 39, 40, 41, 42, 43, 44, 0, 1, 2, 3, 4, 5, 6, 7, 8, 9,
        10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29,
    ],
    [
        29, 30, 31, 32, 33, 34, 35, 36, 37, 38, 39, 40, 41, 42, 43, 44, 0, 1, 2, 3, 4, 5, 6, 7, 8,
        9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28,
    ],
    [
        28, 29, 30, 31, 32, 33, 34, 35, 36, 37, 38, 39, 40, 41, 42, 43, 44, 0, 1, 2, 3, 4, 5, 6, 7,
        8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27,
    ],
    [
        27, 28, 29, 30, 31, 32, 33, 34, 35, 36, 37, 38, 39, 40, 41, 42, 43, 44, 0, 1, 2, 3, 4, 5,
        6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26,
    ],
    [
        26, 27, 28, 29, 30, 31, 32, 33, 34, 35, 36, 37, 38, 39, 40, 41, 42, 43, 44, 0, 1, 2, 3, 4,
        5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25,
    ],
    [
        25, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35, 36, 37, 38, 39, 40, 41, 42, 43, 44, 0, 1, 2, 3,
        4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24,
    ],
    [
        24, 25, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35, 36, 37, 38, 39, 40, 41, 42, 43, 44, 0, 1,
        2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23,
    ],
    [
        23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35, 36, 37, 38, 39, 40, 41, 42, 43, 44, 0,
        1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22,
    ],
    [
        22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35, 36, 37, 38, 39, 40, 41, 42, 43, 44,
        0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21,
    ],
    [
        21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35, 36, 37, 38, 39, 40, 41, 42, 43,
        44, 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
    ],
    [
        20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35, 36, 37, 38, 39, 40, 41, 42,
        43, 44, 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19,
    ],
    [
        19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35, 36, 37, 38, 39, 40, 41,
        42, 43, 44, 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18,
    ],
    [
        18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35, 36, 37, 38, 39, 40,
        41, 42, 43, 44, 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17,
    ],
    [
        17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35, 36, 37, 38, 39,
        40, 41, 42, 43, 44, 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16,
    ],
    [
        16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35, 36, 37, 38,
        39, 40, 41, 42, 43, 44, 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15,
    ],
    [
        15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35, 36, 37,
        38, 39, 40, 41, 42, 43, 44, 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14,
    ],
    [
        14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35, 36,
        37, 38, 39, 40, 41, 42, 43, 44, 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13,
    ],
    [
        13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35,
        36, 37, 38, 39, 40, 41, 42, 43, 44, 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12,
    ],
    [
        12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 33, 34,
        35, 36, 37, 38, 39, 40, 41, 42, 43, 44, 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11,
    ],
    [
        11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 33,
        34, 35, 36, 37, 38, 39, 40, 41, 42, 43, 44, 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10,
    ],
    [
        10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32,
        33, 34, 35, 36, 37, 38, 39, 40, 41, 42, 43, 44, 0, 1, 2, 3, 4, 5, 6, 7, 8, 9,
    ],
    [
        9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31,
        32, 33, 34, 35, 36, 37, 38, 39, 40, 41, 42, 43, 44, 0, 1, 2, 3, 4, 5, 6, 7, 8,
    ],
    [
        8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
        31, 32, 33, 34, 35, 36, 37, 38, 39, 40, 41, 42, 43, 44, 0, 1, 2, 3, 4, 5, 6, 7,
    ],
    [
        7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29,
        30, 31, 32, 33, 34, 35, 36, 37, 38, 39, 40, 41, 42, 43, 44, 0, 1, 2, 3, 4, 5, 6,
    ],
    [
        6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29,
        30, 31, 32, 33, 34, 35, 36, 37, 38, 39, 40, 41, 42, 43, 44, 0, 1, 2, 3, 4, 5,
    ],
    [
        5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28,
        29, 30, 31, 32, 33, 34, 35, 36, 37, 38, 39, 40, 41, 42, 43, 44, 0, 1, 2, 3, 4,
    ],
    [
        4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27,
        28, 29, 30, 31, 32, 33, 34, 35, 36, 37, 38, 39, 40, 41, 42, 43, 44, 0, 1, 2, 3,
    ],
    [
        3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26,
        27, 28, 29, 30, 31, 32, 33, 34, 35, 36, 37, 38, 39, 40, 41, 42, 43, 44, 0, 1, 2,
    ],
    [
        2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26,
        27, 28, 29, 30, 31, 32, 33, 34, 35, 36, 37, 38, 39, 40, 41, 42, 43, 44, 0, 1,
    ],
    [
        1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25,
        26, 27, 28, 29, 30, 31, 32, 33, 34, 35, 36, 37, 38, 39, 40, 41, 42, 43, 44, 0,
    ],
];

/// Ordnung 56: (r,x)*(s,y):=(2r+s,-x+y) mit Rechnung in GF(2^3) x Z_7
const BASE56: [[u8; 56]; 56] = [
    [
        0, 2, 4, 6, 3, 1, 7, 5, 8, 10, 12, 14, 11, 9, 15, 13, 16, 18, 20, 22, 19, 17, 23, 21, 24,
        26, 28, 30, 27, 25, 31, 29, 32, 34, 36, 38, 35, 33, 39, 37, 40, 42, 44, 46, 43, 41, 47, 45,
        48, 50, 52, 54, 51, 49, 55, 53,
    ],
    [
        2, 0, 6, 4, 1, 3, 5, 7, 10, 8, 14, 12, 9, 11, 13, 15, 18, 16, 22, 20, 17, 19, 21, 23, 26,
        24, 30, 28, 25, 27, 29, 31, 34, 32, 38, 36, 33, 35, 37, 39, 42, 40, 46, 44, 41, 43, 45, 47,
        50, 48, 54, 52, 49, 51, 53, 55,
    ],
    [
        4, 6, 0, 2, 7, 5, 3, 1, 12, 14, 8, 10, 15, 13, 11, 9, 20, 22, 16, 18, 23, 21, 19, 17, 28,
        30, 24, 26, 31, 29, 27, 25, 36, 38, 32, 34, 39, 37, 35, 33, 44, 46, 40, 42, 47, 45, 43, 41,
        52, 54, 48, 50, 55, 53, 51, 49,
    ],
    [
        6, 4, 2, 0, 5, 7, 1, 3, 14, 12, 10, 8, 13, 15, 9, 11, 22, 20, 18, 16, 21, 23, 17, 19, 30,
        28, 26, 24, 29, 31, 25, 27, 38, 36, 34, 32, 37, 39, 33, 35, 46, 44, 42, 40, 45, 47, 41, 43,
        54, 52, 50, 48, 53, 55, 49, 51,
    ],
    [
        3, 1, 7, 5, 0, 2, 4, 6, 11, 9, 15, 13, 8, 10, 12, 14, 19, 17, 23, 21, 16, 18, 20, 22, 27,
        25, 31, 29, 24, 26, 28, 30, 35, 33, 39, 37, 32, 34, 36, 38, 43, 41, 47, 45, 40, 42, 44, 46,
        51, 49, 55, 53, 48, 50, 52, 54,
    ],
    [
        1, 3, 5, 7, 2, 0, 6, 4, 9, 11, 13, 15, 10, 8, 14, 12, 17, 19, 21, 23, 18, 16, 22, 20, 25,
        27, 29, 31, 26, 24, 30, 28, 33, 35, 37, 39, 34, 32, 38, 36, 41, 43, 45, 47, 42, 40, 46, 44,
        49, 51, 53, 55, 50, 48, 54, 52,
    ],
    [
        7, 5, 3, 1, 4, 6, 0, 2, 15, 13, 11, 9, 12, 14, 8, 10, 23, 21, 19, 17, 20, 22, 16, 18, 31,
        29, 27, 25, 28, 30, 24, 26, 39, 37, 35, 33, 36, 38, 32, 34, 47, 45, 43, 41, 44, 46, 40, 42,
        55, 53, 51, 49, 52, 54, 48, 50,
    ],
    [
        5, 7, 1, 3, 6, 4, 2, 0, 13, 15, 9, 11, 14, 12, 10, 8, 21, 23, 17, 19, 22, 20, 18, 16, 29,
        31, 25, 27, 30, 28, 26, 24, 37, 39, 33, 35, 38, 36, 34, 32, 45, 47, 41, 43, 46, 44, 42, 40,
        53, 55, 49, 51, 54, 52, 50, 48,
    ],
    [
        48, 50, 52, 54, 51, 49, 55, 53, 0, 2, 4, 6, 3, 1, 7, 5, 8, 10, 12, 14, 11, 9, 15, 13, 16,
        18, 20, 22, 19, 17, 23, 21, 24, 26, 28, 30, 27, 25, 31, 29, 32, 34, 36, 38, 35, 33, 39, 37,
        40, 42, 44, 46, 43, 41, 47, 45,
    ],
    [
        50, 48, 54, 52, 49, 51, 53, 55, 2, 0, 6, 4, 1, 3, 5, 7, 10, 8, 14, 12, 9, 11, 13, 15, 18,
        16, 22, 20, 17, 19, 21, 23, 26, 24, 30, 28, 25, 27, 29, 31, 34, 32, 38, 36, 33, 35, 37, 39,
        42, 40, 46, 44, 41, 43, 45, 47,
    ],
    [
        52, 54, 48, 50, 55, 53, 51, 49, 4, 6, 0, 2, 7, 5, 3, 1, 12, 14, 8, 10, 15, 13, 11, 9, 20,
        22, 16, 18, 23, 21, 19, 17, 28, 30, 24, 26, 31, 29, 27, 25, 36, 38, 32, 34, 39, 37, 35, 33,
        44, 46, 40, 42, 47, 45, 43, 41,
    ],
    [
        54, 52, 50, 48, 53, 55, 49, 51, 6, 4, 2, 0, 5, 7, 1, 3, 14, 12, 10, 8, 13, 15, 9, 11, 22,
        20, 18, 16, 21, 23, 17, 19, 30, 28, 26, 24, 29, 31, 25, 27, 38, 36, 34, 32, 37, 39, 33, 35,
        46, 44, 42, 40, 45, 47, 41, 43,
    ],
    [
        51, 49, 55, 53, 48, 50, 52, 54, 3, 1, 7, 5, 0, 2, 4, 6, 11, 9, 15, 13, 8, 10, 12, 14, 19,
        17, 23, 21, 16, 18, 20, 22, 27, 25, 31, 29, 24, 26, 28, 30, 35, 33, 39, 37, 32, 34, 36, 38,
        43, 41, 47, 45, 40, 42, 44, 46,
    ],
    [
        49, 51, 53, 55, 50, 48, 54, 52, 1, 3, 5, 7, 2, 0, 6, 4, 9, 11, 13, 15, 10, 8, 14, 12, 17,
        19, 21, 23, 18, 16, 22, 20, 25, 27, 29, 31, 26, 24, 30, 28, 33, 35, 37, 39, 34, 32, 38, 36,
        41, 43, 45, 47, 42, 40, 46, 44,
    ],
    [
        55, 53, 51, 49, 52, 54, 48, 50, 7, 5, 3, 1, 4, 6, 0, 2, 15, 13, 11, 9, 12, 14, 8, 10, 23,
        21, 19, 17, 20, 22, 16, 18, 31, 29, 27, 25, 28, 30, 24, 26, 39, 37, 35, 33, 36, 38, 32, 34,
        47, 45, 43, 41, 44, 46, 40, 42,
    ],
    [
        53, 55, 49, 51, 54, 52, 50, 48, 5, 7, 1, 3, 6, 4, 2, 0, 13, 15, 9, 11, 14, 12, 10, 8, 21,
        23, 17, 19, 22, 20, 18, 16, 29, 31, 25, 27, 30, 28, 26, 24, 37, 39, 33, 35, 38, 36, 34, 32,
        45, 47, 41, 43, 46, 44, 42, 40,
    ],
    [
        40, 42, 44, 46, 43, 41, 47, 45, 48, 50, 52, 54, 51, 49, 55, 53, 0, 2, 4, 6, 3, 1, 7, 5, 8,
        10, 12, 14, 11, 9, 15, 13, 16, 18, 20, 22, 19, 17, 23, 21, 24, 26, 28, 30, 27, 25, 31, 29,
        32, 34, 36, 38, 35, 33, 39, 37,
    ],
    [
        42, 40, 46, 44, 41, 43, 45, 47, 50, 48, 54, 52, 49, 51, 53, 55, 2, 0, 6, 4, 1, 3, 5, 7, 10,
        8, 14, 12, 9, 11, 13, 15, 18, 16, 22, 20, 17, 19, 21, 23, 26, 24, 30, 28, 25, 27, 29, 31,
        34, 32, 38, 36, 33, 35, 37, 39,
    ],
    [
        44, 46, 40, 42, 47, 45, 43, 41, 52, 54, 48, 50, 55, 53, 51, 49, 4, 6, 0, 2, 7, 5, 3, 1, 12,
        14, 8, 10, 15, 13, 11, 9, 20, 22, 16, 18, 23, 21, 19, 17, 28, 30, 24, 26, 31, 29, 27, 25,
        36, 38, 32, 34, 39, 37, 35, 33,
    ],
    [
        46, 44, 42, 40, 45, 47, 41, 43, 54, 52, 50, 48, 53, 55, 49, 51, 6, 4, 2, 0, 5, 7, 1, 3, 14,
        12, 10, 8, 13, 15, 9, 11, 22, 20, 18, 16, 21, 23, 17, 19, 30, 28, 26, 24, 29, 31, 25, 27,
        38, 36, 34, 32, 37, 39, 33, 35,
    ],
    [
        43, 41, 47, 45, 40, 42, 44, 46, 51, 49, 55, 53, 48, 50, 52, 54, 3, 1, 7, 5, 0, 2, 4, 6, 11,
        9, 15, 13, 8, 10, 12, 14, 19, 17, 23, 21, 16, 18, 20, 22, 27, 25, 31, 29, 24, 26, 28, 30,
        35, 33, 39, 37, 32, 34, 36, 38,
    ],
    [
        41, 43, 45, 47, 42, 40, 46, 44, 49, 51, 53, 55, 50, 48, 54, 52, 1, 3, 5, 7, 2, 0, 6, 4, 9,
        11, 13, 15, 10, 8, 14, 12, 17, 19, 21, 23, 18, 16, 22, 20, 25, 27, 29, 31, 26, 24, 30, 28,
        33, 35, 37, 39, 34, 32, 38, 36,
    ],
    [
        47, 45, 43, 41, 44, 46, 40, 42, 55, 53, 51, 49, 52, 54, 48, 50, 7, 5, 3, 1, 4, 6, 0, 2, 15,
        13, 11, 9, 12, 14, 8, 10, 23, 21, 19, 17, 20, 22, 16, 18, 31, 29, 27, 25, 28, 30, 24, 26,
        39, 37, 35, 33, 36, 38, 32, 34,
    ],
    [
        45, 47, 41, 43, 46, 44, 42, 40, 53, 55, 49, 51, 54, 52, 50, 48, 5, 7, 1, 3, 6, 4, 2, 0, 13,
        15, 9, 11, 14, 12, 10, 8, 21, 23, 17, 19, 22, 20, 18, 16, 29, 31, 25, 27, 30, 28, 26, 24,
        37, 39, 33, 35, 38, 36, 34, 32,
    ],
    [
        32, 34, 36, 38, 35, 33, 39, 37, 40, 42, 44, 46, 43, 41, 47, 45, 48, 50, 52, 54, 51, 49, 55,
        53, 0, 2, 4, 6, 3, 1, 7, 5, 8, 10, 12, 14, 11, 9, 15, 13, 16, 18, 20, 22, 19, 17, 23, 21,
        24, 26, 28, 30, 27, 25, 31, 29,
    ],
    [
        34, 32, 38, 36, 33, 35, 37, 39, 42, 40, 46, 44, 41, 43, 45, 47, 50, 48, 54, 52, 49, 51, 53,
        55, 2, 0, 6, 4, 1, 3, 5, 7, 10, 8, 14, 12, 9, 11, 13, 15, 18, 16, 22, 20, 17, 19, 21, 23,
        26, 24, 30, 28, 25, 27, 29, 31,
    ],
    [
        36, 38, 32, 34, 39, 37, 35, 33, 44, 46, 40, 42, 47, 45, 43, 41, 52, 54, 48, 50, 55, 53, 51,
        49, 4, 6, 0, 2, 7, 5, 3, 1, 12, 14, 8, 10, 15, 13, 11, 9, 20, 22, 16, 18, 23, 21, 19, 17,
        28, 30, 24, 26, 31, 29, 27, 25,
    ],
    [
        38, 36, 34, 32, 37, 39, 33, 35, 46, 44, 42, 40, 45, 47, 41, 43, 54, 52, 50, 48, 53, 55, 49,
        51, 6, 4, 2, 0, 5, 7, 1, 3, 14, 12, 10, 8, 13, 15, 9, 11, 22, 20, 18, 16, 21, 23, 17, 19,
        30, 28, 26, 24, 29, 31, 25, 27,
    ],
    [
        35, 33, 39, 37, 32, 34, 36, 38, 43, 41, 47, 45, 40, 42, 44, 46, 51, 49, 55, 53, 48, 50, 52,
        54, 3, 1, 7, 5, 0, 2, 4, 6, 11, 9, 15, 13, 8, 10, 12, 14, 19, 17, 23, 21, 16, 18, 20, 22,
        27, 25, 31, 29, 24, 26, 28, 30,
    ],
    [
        33, 35, 37, 39, 34, 32, 38, 36, 41, 43, 45, 47, 42, 40, 46, 44, 49, 51, 53, 55, 50, 48, 54,
        52, 1, 3, 5, 7, 2, 0, 6, 4, 9, 11, 13, 15, 10, 8, 14, 12, 17, 19, 21, 23, 18, 16, 22, 20,
        25, 27, 29, 31, 26, 24, 30, 28,
    ],
    [
        39, 37, 35, 33, 36, 38, 32, 34, 47, 45, 43, 41, 44, 46, 40, 42, 55, 53, 51, 49, 52, 54, 48,
        50, 7, 5, 3, 1, 4, 6, 0, 2, 15, 13, 11, 9, 12, 14, 8, 10, 23, 21, 19, 17, 20, 22, 16, 18,
        31, 29, 27, 25, 28, 30, 24, 26,
    ],
    [
        37, 39, 33, 35, 38, 36, 34, 32, 45, 47, 41, 43, 46, 44, 42, 40, 53, 55, 49, 51, 54, 52, 50,
        48, 5, 7, 1, 3, 6, 4, 2, 0, 13, 15, 9, 11, 14, 12, 10, 8, 21, 23, 17, 19, 22, 20, 18, 16,
        29, 31, 25, 27, 30, 28, 26, 24,
    ],
    [
        24, 26, 28, 30, 27, 25, 31, 29, 32, 34, 36, 38, 35, 33, 39, 37, 40, 42, 44, 46, 43, 41, 47,
        45, 48, 50, 52, 54, 51, 49, 55, 53, 0, 2, 4, 6, 3, 1, 7, 5, 8, 10, 12, 14, 11, 9, 15, 13,
        16, 18, 20, 22, 19, 17, 23, 21,
    ],
    [
        26, 24, 30, 28, 25, 27, 29, 31, 34, 32, 38, 36, 33, 35, 37, 39, 42, 40, 46, 44, 41, 43, 45,
        47, 50, 48, 54, 52, 49, 51, 53, 55, 2, 0, 6, 4, 1, 3, 5, 7, 10, 8, 14, 12, 9, 11, 13, 15,
        18, 16, 22, 20, 17, 19, 21, 23,
    ],
    [
        28, 30, 24, 26, 31, 29, 27, 25, 36, 38, 32, 34, 39, 37, 35, 33, 44, 46, 40, 42, 47, 45, 43,
        41, 52, 54, 48, 50, 55, 53, 51, 49, 4, 6, 0, 2, 7, 5, 3, 1, 12, 14, 8, 10, 15, 13, 11, 9,
        20, 22, 16, 18, 23, 21, 19, 17,
    ],
    [
        30, 28, 26, 24, 29, 31, 25, 27, 38, 36, 34, 32, 37, 39, 33, 35, 46, 44, 42, 40, 45, 47, 41,
        43, 54, 52, 50, 48, 53, 55, 49, 51, 6, 4, 2, 0, 5, 7, 1, 3, 14, 12, 10, 8, 13, 15, 9, 11,
        22, 20, 18, 16, 21, 23, 17, 19,
    ],
    [
        27, 25, 31, 29, 24, 26, 28, 30, 35, 33, 39, 37, 32, 34, 36, 38, 43, 41, 47, 45, 40, 42, 44,
        46, 51, 49, 55, 53, 48, 50, 52, 54, 3, 1, 7, 5, 0, 2, 4, 6, 11, 9, 15, 13, 8, 10, 12, 14,
        19, 17, 23, 21, 16, 18, 20, 22,
    ],
    [
        25, 27, 29, 31, 26, 24, 30, 28, 33, 35, 37, 39, 34, 32, 38, 36, 41, 43, 45, 47, 42, 40, 46,
        44, 49, 51, 53, 55, 50, 48, 54, 52, 1, 3, 5, 7, 2, 0, 6, 4, 9, 11, 13, 15, 10, 8, 14, 12,
        17, 19, 21, 23, 18, 16, 22, 20,
    ],
    [
        31, 29, 27, 25, 28, 30, 24, 26, 39, 37, 35, 33, 36, 38, 32, 34, 47, 45, 43, 41, 44, 46, 40,
        42, 55, 53, 51, 49, 52, 54, 48, 50, 7, 5, 3, 1, 4, 6, 0, 2, 15, 13, 11, 9, 12, 14, 8, 10,
        23, 21, 19, 17, 20, 22, 16, 18,
    ],
    [
        29, 31, 25, 27, 30, 28, 26, 24, 37, 39, 33, 35, 38, 36, 34, 32, 45, 47, 41, 43, 46, 44, 42,
        40, 53, 55, 49, 51, 54, 52, 50, 48, 5, 7, 1, 3, 6, 4, 2, 0, 13, 15, 9, 11, 14, 12, 10, 8,
        21, 23, 17, 19, 22, 20, 18, 16,
    ],
    [
        16, 18, 20, 22, 19, 17, 23, 21, 24, 26, 28, 30, 27, 25, 31, 29, 32, 34, 36, 38, 35, 33, 39,
        37, 40, 42, 44, 46, 43, 41, 47, 45, 48, 50, 52, 54, 51, 49, 55, 53, 0, 2, 4, 6, 3, 1, 7, 5,
        8, 10, 12, 14, 11, 9, 15, 13,
    ],
    [
        18, 16, 22, 20, 17, 19, 21, 23, 26, 24, 30, 28, 25, 27, 29, 31, 34, 32, 38, 36, 33, 35, 37,
        39, 42, 40, 46, 44, 41, 43, 45, 47, 50, 48, 54, 52, 49, 51, 53, 55, 2, 0, 6, 4, 1, 3, 5, 7,
        10, 8, 14, 12, 9, 11, 13, 15,
    ],
    [
        20, 22, 16, 18, 23, 21, 19, 17, 28, 30, 24, 26, 31, 29, 27, 25, 36, 38, 32, 34, 39, 37, 35,
        33, 44, 46, 40, 42, 47, 45, 43, 41, 52, 54, 48, 50, 55, 53, 51, 49, 4, 6, 0, 2, 7, 5, 3, 1,
        12, 14, 8, 10, 15, 13, 11, 9,
    ],
    [
        22, 20, 18, 16, 21, 23, 17, 19, 30, 28, 26, 24, 29, 31, 25, 27, 38, 36, 34, 32, 37, 39, 33,
        35, 46, 44, 42, 40, 45, 47, 41, 43, 54, 52, 50, 48, 53, 55, 49, 51, 6, 4, 2, 0, 5, 7, 1, 3,
        14, 12, 10, 8, 13, 15, 9, 11,
    ],
    [
        19, 17, 23, 21, 16, 18, 20, 22, 27, 25, 31, 29, 24, 26, 28, 30, 35, 33, 39, 37, 32, 34, 36,
        38, 43, 41, 47, 45, 40, 42, 44, 46, 51, 49, 55, 53, 48, 50, 52, 54, 3, 1, 7, 5, 0, 2, 4, 6,
        11, 9, 15, 13, 8, 10, 12, 14,
    ],
    [
        17, 19, 21, 23, 18, 16, 22, 20, 25, 27, 29, 31, 26, 24, 30, 28, 33, 35, 37, 39, 34, 32, 38,
        36, 41, 43, 45, 47, 42, 40, 46, 44, 49, 51, 53, 55, 50, 48, 54, 52, 1, 3, 5, 7, 2, 0, 6, 4,
        9, 11, 13, 15, 10, 8, 14, 12,
    ],
    [
        23, 21, 19, 17, 20, 22, 16, 18, 31, 29, 27, 25, 28, 30, 24, 26, 39, 37, 35, 33, 36, 38, 32,
        34, 47, 45, 43, 41, 44, 46, 40, 42, 55, 53, 51, 49, 52, 54, 48, 50, 7, 5, 3, 1, 4, 6, 0, 2,
        15, 13, 11, 9, 12, 14, 8, 10,
    ],
    [
        21, 23, 17, 19, 22, 20, 18, 16, 29, 31, 25, 27, 30, 28, 26, 24, 37, 39, 33, 35, 38, 36, 34,
        32, 45, 47, 41, 43, 46, 44, 42, 40, 53, 55, 49, 51, 54, 52, 50, 48, 5, 7, 1, 3, 6, 4, 2, 0,
        13, 15, 9, 11, 14, 12, 10, 8,
    ],
    [
        8, 10, 12, 14, 11, 9, 15, 13, 16, 18, 20, 22, 19, 17, 23, 21, 24, 26, 28, 30, 27, 25, 31,
        29, 32, 34, 36, 38, 35, 33, 39, 37, 40, 42, 44, 46, 43, 41, 47, 45, 48, 50, 52, 54, 51, 49,
        55, 53, 0, 2, 4, 6, 3, 1, 7, 5,
    ],
    [
        10, 8, 14, 12, 9, 11, 13, 15, 18, 16, 22, 20, 17, 19, 21, 23, 26, 24, 30, 28, 25, 27, 29,
        31, 34, 32, 38, 36, 33, 35, 37, 39, 42, 40, 46, 44, 41, 43, 45, 47, 50, 48, 54, 52, 49, 51,
        53, 55, 2, 0, 6, 4, 1, 3, 5, 7,
    ],
    [
        12, 14, 8, 10, 15, 13, 11, 9, 20, 22, 16, 18, 23, 21, 19, 17, 28, 30, 24, 26, 31, 29, 27,
        25, 36, 38, 32, 34, 39, 37, 35, 33, 44, 46, 40, 42, 47, 45, 43, 41, 52, 54, 48, 50, 55, 53,
        51, 49, 4, 6, 0, 2, 7, 5, 3, 1,
    ],
    [
        14, 12, 10, 8, 13, 15, 9, 11, 22, 20, 18, 16, 21, 23, 17, 19, 30, 28, 26, 24, 29, 31, 25,
        27, 38, 36, 34, 32, 37, 39, 33, 35, 46, 44, 42, 40, 45, 47, 41, 43, 54, 52, 50, 48, 53, 55,
        49, 51, 6, 4, 2, 0, 5, 7, 1, 3,
    ],
    [
        11, 9, 15, 13, 8, 10, 12, 14, 19, 17, 23, 21, 16, 18, 20, 22, 27, 25, 31, 29, 24, 26, 28,
        30, 35, 33, 39, 37, 32, 34, 36, 38, 43, 41, 47, 45, 40, 42, 44, 46, 51, 49, 55, 53, 48, 50,
        52, 54, 3, 1, 7, 5, 0, 2, 4, 6,
    ],
    [
        9, 11, 13, 15, 10, 8, 14, 12, 17, 19, 21, 23, 18, 16, 22, 20, 25, 27, 29, 31, 26, 24, 30,
        28, 33, 35, 37, 39, 34, 32, 38, 36, 41, 43, 45, 47, 42, 40, 46, 44, 49, 51, 53, 55, 50, 48,
        54, 52, 1, 3, 5, 7, 2, 0, 6, 4,
    ],
    [
        15, 13, 11, 9, 12, 14, 8, 10, 23, 21, 19, 17, 20, 22, 16, 18, 31, 29, 27, 25, 28, 30, 24,
        26, 39, 37, 35, 33, 36, 38, 32, 34, 47, 45, 43, 41, 44, 46, 40, 42, 55, 53, 51, 49, 52, 54,
        48, 50, 7, 5, 3, 1, 4, 6, 0, 2,
    ],
    [
        13, 15, 9, 11, 14, 12, 10, 8, 21, 23, 17, 19, 22, 20, 18, 16, 29, 31, 25, 27, 30, 28, 26,
        24, 37, 39, 33, 35, 38, 36, 34, 32, 45, 47, 41, 43, 46, 44, 42, 40, 53, 55, 49, 51, 54, 52,
        50, 48, 5, 7, 1, 3, 6, 4, 2, 0,
    ],
];

/// Ordnung 58: singulär direktes Produkt 3·19+1
const BASE58: [[u8; 58]; 58] = [
    [
        0, 2, 3, 1, 5, 6, 4, 8, 9, 7, 11, 12, 10, 14, 15, 13, 17, 18, 16, 20, 21, 19, 23, 24, 22,
        26, 27, 25, 29, 30, 28, 32, 33, 31, 35, 36, 34, 38, 39, 37, 41, 42, 40, 44, 45, 43, 47, 48,
        46, 50, 51, 49, 53, 54, 52, 56, 57, 55,
    ],
    [
        2, 0, 1, 3, 8, 9, 7, 14, 15, 13, 20, 21, 19, 26, 27, 25, 32, 33, 31, 38, 39, 37, 44, 45,
        43, 50, 51, 49, 56, 57, 55, 5, 6, 4, 11, 12, 10, 17, 18, 16, 23, 24, 22, 29, 30, 28, 35,
        36, 34, 41, 42, 40, 47, 48, 46, 53, 54, 52,
    ],
    [
        3, 1, 0, 2, 7, 8, 9, 13, 14, 15, 19, 20, 21, 25, 26, 27, 31, 32, 33, 37, 38, 39, 43, 44,
        45, 49, 50, 51, 55, 56, 57, 4, 5, 6, 10, 11, 12, 16, 17, 18, 22, 23, 24, 28, 29, 30, 34,
        35, 36, 40, 41, 42, 46, 47, 48, 52, 53, 54,
    ],
    [
        1, 3, 2, 0, 9, 7, 8, 15, 13, 14, 21, 19, 20, 27, 25, 26, 33, 31, 32, 39, 37, 38, 45, 43,
        44, 51, 49, 50, 57, 55, 56, 6, 4, 5, 12, 10, 11, 18, 16, 17, 24, 22, 23, 30, 28, 29, 36,
        34, 35, 42, 40, 41, 48, 46, 47, 54, 52, 53,
    ],
    [
        5, 56, 57, 55, 0, 4, 6, 11, 12, 10, 17, 18, 16, 23, 24, 22, 29, 30, 28, 35, 36, 34, 41, 42,
        40, 47, 48, 46, 53, 54, 52, 2, 3, 1, 8, 9, 7, 14, 15, 13, 20, 21, 19, 26, 27, 25, 32, 33,
        31, 38, 39, 37, 44, 45, 43, 50, 51, 49,
    ],
    [
        6, 55, 56, 57, 4, 0, 5, 10, 11, 12, 16, 17, 18, 22, 23, 24, 28, 29, 30, 34, 35, 36, 40, 41,
        42, 46, 47, 48, 52, 53, 54, 1, 2, 3, 7, 8, 9, 13, 14, 15, 19, 20, 21, 25, 26, 27, 31, 32,
        33, 37, 38, 39, 43, 44, 45, 49, 50, 51,
    ],
    [
        4, 57, 55, 56, 6, 5, 0, 12, 10, 11, 18, 16, 17, 24, 22, 23, 30, 28, 29, 36, 34, 35, 42, 40,
        41, 48, 46, 47, 54, 52, 53, 3, 1, 2, 9, 7, 8, 15, 13, 14, 21, 19, 20, 27, 25, 26, 33, 31,
        32, 39, 37, 38, 45, 43, 44, 51, 49, 50,
    ],
    [
        8, 53, 54, 52, 2, 3, 1, 0, 7, 9, 14, 15, 13, 20, 21, 19, 26, 27, 25, 32, 33, 31, 38, 39,
        37, 44, 45, 43, 50, 51, 49, 56, 57, 55, 5, 6, 4, 11, 12, 10, 17, 18, 16, 23, 24, 22, 29,
        30, 28, 35, 36, 34, 41, 42, 40, 47, 48, 46,
    ],
    [
        9, 52, 53, 54, 1, 2, 3, 7, 0, 8, 13, 14, 15, 19, 20, 21, 25, 26, 27, 31, 32, 33, 37, 38,
        39, 43, 44, 45, 49, 50, 51, 55, 56, 57, 4, 5, 6, 10, 11, 12, 16, 17, 18, 22, 23, 24, 28,
        29, 30, 34, 35, 36, 40, 41, 42, 46, 47, 48,
    ],
    [
        7, 54, 52, 53, 3, 1, 2, 9, 8, 0, 15, 13, 14, 21, 19, 20, 27, 25, 26, 33, 31, 32, 39, 37,
        38, 45, 43, 44, 51, 49, 50, 57, 55, 56, 6, 4, 5, 12, 10, 11, 18, 16, 17, 24, 22, 23, 30,
        28, 29, 36, 34, 35, 42, 40, 41, 48, 46, 47,
    ],
    [
        11, 50, 51, 49, 56, 57, 55, 5, 6, 4, 0, 10, 12, 17, 18, 16, 23, 24, 22, 29, 30, 28, 35, 36,
        34, 41, 42, 40, 47, 48, 46, 53, 54, 52, 2, 3, 1, 8, 9, 7, 14, 15, 13, 20, 21, 19, 26, 27,
        25, 32, 33, 31, 38, 39, 37, 44, 45, 43,
    ],
    [
        12, 49, 50, 51, 55, 56, 57, 4, 5, 6, 10, 0, 11, 16, 17, 18, 22, 23, 24, 28, 29, 30, 34, 35,
        36, 40, 41, 42, 46, 47, 48, 52, 53, 54, 1, 2, 3, 7, 8, 9, 13, 14, 15, 19, 20, 21, 25, 26,
        27, 31, 32, 33, 37, 38, 39, 43, 44, 45,
    ],
    [
        10, 51, 49, 50, 57, 55, 56, 6, 4, 5, 12, 11, 0, 18, 16, 17, 24, 22, 23, 30, 28, 29, 36, 34,
        35, 42, 40, 41, 48, 46, 47, 54, 52, 53, 3, 1, 2, 9, 7, 8, 15, 13, 14, 21, 19, 20, 27, 25,
        26, 33, 31, 32, 39, 37, 38, 45, 43, 44,
    ],
    [
        14, 47, 48, 46, 53, 54, 52, 2, 3, 1, 8, 9, 7, 0, 13, 15, 20, 21, 19, 26, 27, 25, 32, 33,
        31, 38, 39, 37, 44, 45, 43, 50, 51, 49, 56, 57, 55, 5, 6, 4, 11, 12, 10, 17, 18, 16, 23,
        24, 22, 29, 30, 28, 35, 36, 34, 41, 42, 40,
    ],
    [
        15, 46, 47, 48, 52, 53, 54, 1, 2, 3, 7, 8, 9, 13, 0, 14, 19, 20, 21, 25, 26, 27, 31, 32,
        33, 37, 38, 39, 43, 44, 45, 49, 50, 51, 55, 56, 57, 4, 5, 6, 10, 11, 12, 16, 17, 18, 22,
        23, 24, 28, 29, 30, 34, 35, 36, 40, 41, 42,
    ],
    [
        13, 48, 46, 47, 54, 52, 53, 3, 1, 2, 9, 7, 8, 15, 14, 0, 21, 19, 20, 27, 25, 26, 33, 31,
        32, 39, 37, 38, 45, 43, 44, 51, 49, 50, 57, 55, 56, 6, 4, 5, 12, 10, 11, 18, 16, 17, 24,
        22, 23, 30, 28, 29, 36, 34, 35, 42, 40, 41,
    ],
    [
        17, 44, 45, 43, 50, 51, 49, 56, 57, 55, 5, 6, 4, 11, 12, 10, 0, 16, 18, 23, 24, 22, 29, 30,
        28, 35, 36, 34, 41, 42, 40, 47, 48, 46, 53, 54, 52, 2, 3, 1, 8, 9, 7, 14, 15, 13, 20, 21,
        19, 26, 27, 25, 32, 33, 31, 38, 39, 37,
    ],
    [
        18, 43, 44, 45, 49, 50, 51, 55, 56, 57, 4, 5, 6, 10, 11, 12, 16, 0, 17, 22, 23, 24, 28, 29,
        30, 34, 35, 36, 40, 41, 42, 46, 47, 48, 52, 53, 54, 1, 2, 3, 7, 8, 9, 13, 14, 15, 19, 20,
        21, 25, 26, 27, 31, 32, 33, 37, 38, 39,
    ],
    [
        16, 45, 43, 44, 51, 49, 50, 57, 55, 56, 6, 4, 5, 12, 10, 11, 18, 17, 0, 24, 22, 23, 30, 28,
        29, 36, 34, 35, 42, 40, 41, 48, 46, 47, 54, 52, 53, 3, 1, 2, 9, 7, 8, 15, 13, 14, 21, 19,
        20, 27, 25, 26, 33, 31, 32, 39, 37, 38,
    ],
    [
        20, 41, 42, 40, 47, 48, 46, 53, 54, 52, 2, 3, 1, 8, 9, 7, 14, 15, 13, 0, 19, 21, 26, 27,
        25, 32, 33, 31, 38, 39, 37, 44, 45, 43, 50, 51, 49, 56, 57, 55, 5, 6, 4, 11, 12, 10, 17,
        18, 16, 23, 24, 22, 29, 30, 28, 35, 36, 34,
    ],
    [
        21, 40, 41, 42, 46, 47, 48, 52, 53, 54, 1, 2, 3, 7, 8, 9, 13, 14, 15, 19, 0, 20, 25, 26,
        27, 31, 32, 33, 37, 38, 39, 43, 44, 45, 49, 50, 51, 55, 56, 57, 4, 5, 6, 10, 11, 12, 16,
        17, 18, 22, 23, 24, 28, 29, 30, 34, 35, 36,
    ],
    [
        19, 42, 40, 41, 48, 46, 47, 54, 52, 53, 3, 1, 2, 9, 7, 8, 15, 13, 14, 21, 20, 0, 27, 25,
        26, 33, 31, 32, 39, 37, 38, 45, 43, 44, 51, 49, 50, 57, 55, 56, 6, 4, 5, 12, 10, 11, 18,
        16, 17, 24, 22, 23, 30, 28, 29, 36, 34, 35,
    ],
    [
        23, 38, 39, 37, 44, 45, 43, 50, 51, 49, 56, 57, 55, 5, 6, 4, 11, 12, 10, 17, 18, 16, 0, 22,
        24, 29, 30, 28, 35, 36, 34, 41, 42, 40, 47, 48, 46, 53, 54, 52, 2, 3, 1, 8, 9, 7, 14, 15,
        13, 20, 21, 19, 26, 27, 25, 32, 33, 31,
    ],
    [
        24, 37, 38, 39, 43, 44, 45, 49, 50, 51, 55, 56, 57, 4, 5, 6, 10, 11, 12, 16, 17, 18, 22, 0,
        23, 28, 29, 30, 34, 35, 36, 40, 41, 42, 46, 47, 48, 52, 53, 54, 1, 2, 3, 7, 8, 9, 13, 14,
        15, 19, 20, 21, 25, 26, 27, 31, 32, 33,
    ],
    [
        22, 39, 37, 38, 45, 43, 44, 51, 49, 50, 57, 55, 56, 6, 4, 5, 12, 10, 11, 18, 16, 17, 24,
        23, 0, 30, 28, 29, 36, 34, 35, 42, 40, 41, 48, 46, 47, 54, 52, 53, 3, 1, 2, 9, 7, 8, 15,
        13, 14, 21, 19, 20, 27, 25, 26, 33, 31, 32,
    ],
    [
        26, 35, 36, 34, 41, 42, 40, 47, 48, 46, 53, 54, 52, 2, 3, 1, 8, 9, 7, 14, 15, 13, 20, 21,
        19, 0, 25, 27, 32, 33, 31, 38, 39, 37, 44, 45, 43, 50, 51, 49, 56, 57, 55, 5, 6, 4, 11, 12,
        10, 17, 18, 16, 23, 24, 22, 29, 30, 28,
    ],
    [
        27, 34, 35, 36, 40, 41, 42, 46, 47, 48, 52, 53, 54, 1, 2, 3, 7, 8, 9, 13, 14, 15, 19, 20,
        21, 25, 0, 26, 31, 32, 33, 37, 38, 39, 43, 44, 45, 49, 50, 51, 55, 56, 57, 4, 5, 6, 10, 11,
        12, 16, 17, 18, 22, 23, 24, 28, 29, 30,
    ],
    [
        25, 36, 34, 35, 42, 40, 41, 48, 46, 47, 54, 52, 53, 3, 1, 2, 9, 7, 8, 15, 13, 14, 21, 19,
        20, 27, 26, 0, 33, 31, 32, 39, 37, 38, 45, 43, 44, 51, 49, 50, 57, 55, 56, 6, 4, 5, 12, 10,
        11, 18, 16, 17, 24, 22, 23, 30, 28, 29,
    ],
    [
        29, 32, 33, 31, 38, 39, 37, 44, 45, 43, 50, 51, 49, 56, 57, 55, 5, 6, 4, 11, 12, 10, 17,
        18, 16, 23, 24, 22, 0, 28, 30, 35, 36, 34, 41, 42, 40, 47, 48, 46, 53, 54, 52, 2, 3, 1, 8,
        9, 7, 14, 15, 13, 20, 21, 19, 26, 27, 25,
    ],
    [
        30, 31, 32, 33, 37, 38, 39, 43, 44, 45, 49, 50, 51, 55, 56, 57, 4, 5, 6, 10, 11, 12, 16,
        17, 18, 22, 23, 24, 28, 0, 29, 34, 35, 36, 40, 41, 42, 46, 47, 48, 52, 53, 54, 1, 2, 3, 7,
        8, 9, 13, 14, 15, 19, 20, 21, 25, 26, 27,
    ],
    [
        28, 33, 31, 32, 39, 37, 38, 45, 43, 44, 51, 49, 50, 57, 55, 56, 6, 4, 5, 12, 10, 11, 18,
        16, 17, 24, 22, 23, 30, 29, 0, 36, 34, 35, 42, 40, 41, 48, 46, 47, 54, 52, 53, 3, 1, 2, 9,
        7, 8, 15, 13, 14, 21, 19, 20, 27, 25, 26,
    ],
    [
        32, 29, 30, 28, 35, 36, 34, 41, 42, 40, 47, 48, 46, 53, 54, 52, 2, 3, 1, 8, 9, 7, 14, 15,
        13, 20, 21, 19, 26, 27, 25, 0, 31, 33, 38, 39, 37, 44, 45, 43, 50, 51, 49, 56, 57, 55, 5,
        6, 4, 11, 12, 10, 17, 18, 16, 23, 24, 22,
    ],
    [
        33, 28, 29, 30, 34, 35, 36, 40, 41, 42, 46, 47, 48, 52, 53, 54, 1, 2, 3, 7, 8, 9, 13, 14,
        15, 19, 20, 21, 25, 26, 27, 31, 0, 32, 37, 38, 39, 43, 44, 45, 49, 50, 51, 55, 56, 57, 4,
        5, 6, 10, 11, 12, 16, 17, 18, 22, 23, 24,
    ],
    [
        31, 30, 28, 29, 36, 34, 35, 42, 40, 41, 48, 46, 47, 54, 52, 53, 3, 1, 2, 9, 7, 8, 15, 13,
        14, 21, 19, 20, 27, 25, 26, 33, 32, 0, 39, 37, 38, 45, 43, 44, 51, 49, 50, 57, 55, 56, 6,
        4, 5, 12, 10, 11, 18, 16, 17, 24, 22, 23,
    ],
    [
        35, 26, 27, 25, 32, 33, 31, 38, 39, 37, 44, 45, 43, 50, 51, 49, 56, 57, 55, 5, 6, 4, 11,
        12, 10, 17, 18, 16, 23, 24, 22, 29, 30, 28, 0, 34, 36, 41, 42, 40, 47, 48, 46, 53, 54, 52,
        2, 3, 1, 8, 9, 7, 14, 15, 13, 20, 21, 19,
    ],
    [
        36, 25, 26, 27, 31, 32, 33, 37, 38, 39, 43, 44, 45, 49, 50, 51, 55, 56, 57, 4, 5, 6, 10,
        11, 12, 16, 17, 18, 22, 23, 24, 28, 29, 30, 34, 0, 35, 40, 41, 42, 46, 47, 48, 52, 53, 54,
        1, 2, 3, 7, 8, 9, 13, 14, 15, 19, 20, 21,
    ],
    [
        34, 27, 25, 26, 33, 31, 32, 39, 37, 38, 45, 43, 44, 51, 49, 50, 57, 55, 56, 6, 4, 5, 12,
        10, 11, 18, 16, 17, 24, 22, 23, 30, 28, 29, 36, 35, 0, 42, 40, 41, 48, 46, 47, 54, 52, 53,
        3, 1, 2, 9, 7, 8, 15, 13, 14, 21, 19, 20,
    ],
    [
        38, 23, 24, 22, 29, 30, 28, 35, 36, 34, 41, 42, 40, 47, 48, 46, 53, 54, 52, 2, 3, 1, 8, 9,
        7, 14, 15, 13, 20, 21, 19, 26, 27, 25, 32, 33, 31, 0, 37, 39, 44, 45, 43, 50, 51, 49, 56,
        57, 55, 5, 6, 4, 11, 12, 10, 17, 18, 16,
    ],
    [
        39, 22, 23, 24, 28, 29, 30, 34, 35, 36, 40, 41, 42, 46, 47, 48, 52, 53, 54, 1, 2, 3, 7, 8,
        9, 13, 14, 15, 19, 20, 21, 25, 26, 27, 31, 32, 33, 37, 0, 38, 43, 44, 45, 49, 50, 51, 55,
        56, 57, 4, 5, 6, 10, 11, 12, 16, 17, 18,
    ],
    [
        37, 24, 22, 23, 30, 28, 29, 36, 34, 35, 42, 40, 41, 48, 46, 47, 54, 52, 53, 3, 1, 2, 9, 7,
        8, 15, 13, 14, 21, 19, 20, 27, 25, 26, 33, 31, 32, 39, 38, 0, 45, 43, 44, 51, 49, 50, 57,
        55, 56, 6, 4, 5, 12, 10, 11, 18, 16, 17,
    ],
    [
        41, 20, 21, 19, 26, 27, 25, 32, 33, 31, 38, 39, 37, 44, 45, 43, 50, 51, 49, 56, 57, 55, 5,
        6, 4, 11, 12, 10, 17, 18, 16, 23, 24, 22, 29, 30, 28, 35, 36, 34, 0, 40, 42, 47, 48, 46,
        53, 54, 52, 2, 3, 1, 8, 9, 7, 14, 15, 13,
    ],
    [
        42, 19, 20, 21, 25, 26, 27, 31, 32, 33, 37, 38, 39, 43, 44, 45, 49, 50, 51, 55, 56, 57, 4,
        5, 6, 10, 11, 12, 16, 17, 18, 22, 23, 24, 28, 29, 30, 34, 35, 36, 40, 0, 41, 46, 47, 48,
        52, 53, 54, 1, 2, 3, 7, 8, 9, 13, 14, 15,
    ],
    [
        40, 21, 19, 20, 27, 25, 26, 33, 31, 32, 39, 37, 38, 45, 43, 44, 51, 49, 50, 57, 55, 56, 6,
        4, 5, 12, 10, 11, 18, 16, 17, 24, 22, 23, 30, 28, 29, 36, 34, 35, 42, 41, 0, 48, 46, 47,
        54, 52, 53, 3, 1, 2, 9, 7, 8, 15, 13, 14,
    ],
    [
        44, 17, 18, 16, 23, 24, 22, 29, 30, 28, 35, 36, 34, 41, 42, 40, 47, 48, 46, 53, 54, 52, 2,
        3, 1, 8, 9, 7, 14, 15, 13, 20, 21, 19, 26, 27, 25, 32, 33, 31, 38, 39, 37, 0, 43, 45, 50,
        51, 49, 56, 57, 55, 5, 6, 4, 11, 12, 10,
    ],
    [
        45, 16, 17, 18, 22, 23, 24, 28, 29, 30, 34, 35, 36, 40, 41, 42, 46, 47, 48, 52, 53, 54, 1,
        2, 3, 7, 8, 9, 13, 14, 15, 19, 20, 21, 25, 26, 27, 31, 32, 33, 37, 38, 39, 43, 0, 44, 49,
        50, 51, 55, 56, 57, 4, 5, 6, 10, 11, 12,
    ],
    [
        43, 18, 16, 17, 24, 22, 23, 30, 28, 29, 36, 34, 35, 42, 40, 41, 48, 46, 47, 54, 52, 53, 3,
        1, 2, 9, 7, 8, 15, 13, 14, 21, 19, 20, 27, 25, 26, 33, 31, 32, 39, 37, 38, 45, 44, 0, 51,
        49, 50, 57, 55, 56, 6, 4, 5, 12, 10, 11,
    ],
    [
        47, 14, 15, 13, 20, 21, 19, 26, 27, 25, 32, 33, 31, 38, 39, 37, 44, 45, 43, 50, 51, 49, 56,
        57, 55, 5, 6, 4, 11, 12, 10, 17, 18, 16, 23, 24, 22, 29, 30, 28, 35, 36, 34, 41, 42, 40, 0,
        46, 48, 53, 54, 52, 2, 3, 1, 8, 9, 7,
    ],
    [
        48, 13, 14, 15, 19, 20, 21, 25, 26, 27, 31, 32, 33, 37, 38, 39, 43, 44, 45, 49, 50, 51, 55,
        56, 57, 4, 5, 6, 10, 11, 12, 16, 17, 18, 22, 23, 24, 28, 29, 30, 34, 35, 36, 40, 41, 42,
        46, 0, 47, 52, 53, 54, 1, 2, 3, 7, 8, 9,
    ],
    [
        46, 15, 13, 14, 21, 19, 20, 27, 25, 26, 33, 31, 32, 39, 37, 38, 45, 43, 44, 51, 49, 50, 57,
        55, 56, 6, 4, 5, 12, 10, 11, 18, 16, 17, 24, 22, 23, 30, 28, 29, 36, 34, 35, 42, 40, 41,
        48, 47, 0, 54, 52, 53, 3, 1, 2, 9, 7, 8,
    ],
    [
        50, 11, 12, 10, 17, 18, 16, 23, 24, 22, 29, 30, 28, 35, 36, 34, 41, 42, 40, 47, 48, 46, 53,
        54, 52, 2, 3, 1, 8, 9, 7, 14, 15, 13, 20, 21, 19, 26, 27, 25, 32, 33, 31, 38, 39, 37, 44,
        45, 43, 0, 49, 51, 56, 57, 55, 5, 6, 4,
    ],
    [
        51, 10, 11, 12, 16, 17, 18, 22, 23, 24, 28, 29, 30, 34, 35, 36, 40, 41, 42, 46, 47, 48, 52,
        53, 54, 1, 2, 3, 7, 8, 9, 13, 14, 15, 19, 20, 21, 25, 26, 27, 31, 32, 33, 37, 38, 39, 43,
        44, 45, 49, 0, 50, 55, 56, 57, 4, 5, 6,
    ],
    [
        49, 12, 10, 11, 18, 16, 17, 24, 22, 23, 30, 28, 29, 36, 34, 35, 42, 40, 41, 48, 46, 47, 54,
        52, 53, 3, 1, 2, 9, 7, 8, 15, 13, 14, 21, 19, 20, 27, 25, 26, 33, 31, 32, 39, 37, 38, 45,
        43, 44, 51, 50, 0, 57, 55, 56, 6, 4, 5,
    ],
    [
        53, 8, 9, 7, 14, 15, 13, 20, 21, 19, 26, 27, 25, 32, 33, 31, 38, 39, 37, 44, 45, 43, 50,
        51, 49, 56, 57, 55, 5, 6, 4, 11, 12, 10, 17, 18, 16, 23, 24, 22, 29, 30, 28, 35, 36, 34,
        41, 42, 40, 47, 48, 46, 0, 52, 54, 2, 3, 1,
    ],
    [
        54, 7, 8, 9, 13, 14, 15, 19, 20, 21, 25, 26, 27, 31, 32, 33, 37, 38, 39, 43, 44, 45, 49,
        50, 51, 55, 56, 57, 4, 5, 6, 10, 11, 12, 16, 17, 18, 22, 23, 24, 28, 29, 30, 34, 35, 36,
        40, 41, 42, 46, 47, 48, 52, 0, 53, 1, 2, 3,
    ],
    [
        52, 9, 7, 8, 15, 13, 14, 21, 19, 20, 27, 25, 26, 33, 31, 32, 39, 37, 38, 45, 43, 44, 51,
        49, 50, 57, 55, 56, 6, 4, 5, 12, 10, 11, 18, 16, 17, 24, 22, 23, 30, 28, 29, 36, 34, 35,
        42, 40, 41, 48, 46, 47, 54, 53, 0, 3, 1, 2,
    ],
    [
        56, 5, 6, 4, 11, 12, 10, 17, 18, 16, 23, 24, 22, 29, 30, 28, 35, 36, 34, 41, 42, 40, 47,
        48, 46, 53, 54, 52, 2, 3, 1, 8, 9, 7, 14, 15, 13, 20, 21, 19, 26, 27, 25, 32, 33, 31, 38,
        39, 37, 44, 45, 43, 50, 51, 49, 0, 55, 57,
    ],
    [
        57, 4, 5, 6, 10, 11, 12, 16, 17, 18, 22, 23, 24, 28, 29, 30, 34, 35, 36, 40, 41, 42, 46,
        47, 48, 52, 53, 54, 1, 2, 3, 7, 8, 9, 13, 14, 15, 19, 20, 21, 25, 26, 27, 31, 32, 33, 37,
        38, 39, 43, 44, 45, 49, 50, 51, 55, 0, 56,
    ],
    [
        55, 6, 4, 5, 12, 10, 11, 18, 16, 17, 24, 22, 23, 30, 28, 29, 36, 34, 35, 42, 40, 41, 48,
        46, 47, 54, 52, 53, 3, 1, 2, 9, 7, 8, 15, 13, 14, 21, 19, 20, 27, 25, 26, 33, 31, 32, 39,
        37, 38, 45, 43, 44, 51, 49, 50, 57, 56, 0,
    ],
];

/// Ordnung 62: Insertion-Konstruktion 61+1
const BASE62: [[u8; 62]; 62] = [
    [
        0, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25,
        26, 27, 28, 29, 30, 31, 32, 33, 34, 35, 36, 37, 38, 39, 40, 41, 42, 43, 44, 45, 46, 47, 48,
        49, 50, 51, 52, 53, 54, 55, 56, 57, 58, 59, 60, 61, 1,
    ],
    [
        2, 0, 50, 37, 24, 11, 59, 46, 33, 20, 7, 55, 42, 29, 16, 3, 51, 38, 25, 12, 60, 47, 34, 21,
        8, 56, 43, 30, 17, 4, 52, 39, 26, 13, 61, 48, 35, 22, 9, 57, 44, 31, 18, 5, 53, 40, 27, 14,
        1, 49, 36, 23, 10, 58, 45, 32, 19, 6, 54, 41, 28, 15,
    ],
    [
        3, 16, 0, 51, 38, 25, 12, 60, 47, 34, 21, 8, 56, 43, 30, 17, 4, 52, 39, 26, 13, 61, 48, 35,
        22, 9, 57, 44, 31, 18, 5, 53, 40, 27, 14, 1, 49, 36, 23, 10, 58, 45, 32, 19, 6, 54, 41, 28,
        15, 2, 50, 37, 24, 11, 59, 46, 33, 20, 7, 55, 42, 29,
    ],
    [
        4, 30, 17, 0, 52, 39, 26, 13, 61, 48, 35, 22, 9, 57, 44, 31, 18, 5, 53, 40, 27, 14, 1, 49,
        36, 23, 10, 58, 45, 32, 19, 6, 54, 41, 28, 15, 2, 50, 37, 24, 11, 59, 46, 33, 20, 7, 55,
        42, 29, 16, 3, 51, 38, 25, 12, 60, 47, 34, 21, 8, 56, 43,
    ],
    [
        5, 44, 31, 18, 0, 53, 40, 27, 14, 1, 49, 36, 23, 10, 58, 45, 32, 19, 6, 54, 41, 28, 15, 2,
        50, 37, 24, 11, 59, 46, 33, 20, 7, 55, 42, 29, 16, 3, 51, 38, 25, 12, 60, 47, 34, 21, 8,
        56, 43, 30, 17, 4, 52, 39, 26, 13, 61, 48, 35, 22, 9, 57,
    ],
    [
        6, 58, 45, 32, 19, 0, 54, 41, 28, 15, 2, 50, 37, 24, 11, 59, 46, 33, 20, 7, 55, 42, 29, 16,
        3, 51, 38, 25, 12, 60, 47, 34, 21, 8, 56, 43, 30, 17, 4, 52, 39, 26, 13, 61, 48, 35, 22, 9,
        57, 44, 31, 18, 5, 53, 40, 27, 14, 1, 49, 36, 23, 10,
    ],
    [
        7, 11, 59, 46, 33, 20, 0, 55, 42, 29, 16, 3, 51, 38, 25, 12, 60, 47, 34, 21, 8, 56, 43, 30,
        17, 4, 52, 39, 26, 13, 61, 48, 35, 22, 9, 57, 44, 31, 18, 5, 53, 40, 27, 14, 1, 49, 36, 23,
        10, 58, 45, 32, 19, 6, 54, 41, 28, 15, 2, 50, 37, 24,
    ],
    [
        8, 25, 12, 60, 47, 34, 21, 0, 56, 43, 30, 17, 4, 52, 39, 26, 13, 61, 48, 35, 22, 9, 57, 44,
        31, 18, 5, 53, 40, 27, 14, 1, 49, 36, 23, 10, 58, 45, 32, 19, 6, 54, 41, 28, 15, 2, 50, 37,
        24, 11, 59, 46, 33, 20, 7, 55, 42, 29, 16, 3, 51, 38,
    ],
    [
        9, 39, 26, 13, 61, 48, 35, 22, 0, 57, 44, 31, 18, 5, 53, 40, 27, 14, 1, 49, 36, 23, 10, 58,
        45, 32, 19, 6, 54, 41, 28, 15, 2, 50, 37, 24, 11, 59, 46, 33, 20, 7, 55, 42, 29, 16, 3, 51,
        38, 25, 12, 60, 47, 34, 21, 8, 56, 43, 30, 17, 4, 52,
    ],
    [
        10, 53, 40, 27, 14, 1, 49, 36, 23, 0, 58, 45, 32, 19, 6, 54, 41, 28, 15, 2, 50, 37, 24, 11,
        59, 46, 33, 20, 7, 55, 42, 29, 16, 3, 51, 38, 25, 12, 60, 47, 34, 21, 8, 56, 43, 30, 17, 4,
        52, 39, 26, 13, 61, 48, 35, 22, 9, 57, 44, 31, 18, 5,
    ],
    [
        11, 6, 54, 41, 28, 15, 2, 50, 37, 24, 0, 59, 46, 33, 20, 7, 55, 42, 29, 16, 3, 51, 38, 25,
        12, 60, 47, 34, 21, 8, 56, 43, 30, 17, 4, 52, 39, 26, 13, 61, 48, 35, 22, 9, 57, 44, 31,
        18, 5, 53, 40, 27, 14, 1, 49, 36, 23, 10, 58, 45, 32, 19,
    ],
    [
        12, 20, 7, 55, 42, 29, 16, 3, 51, 38, 25, 0, 60, 47, 34, 21, 8, 56, 43, 30, 17, 4, 52, 39,
        26, 13, 61, 48, 35, 22, 9, 57, 44, 31, 18, 5, 53, 40, 27, 14, 1, 49, 36, 23, 10, 58, 45,
        32, 19, 6, 54, 41, 28, 15, 2, 50, 37, 24, 11, 59, 46, 33,
    ],
    [
        13, 34, 21, 8, 56, 43, 30, 17, 4, 52, 39, 26, 0, 61, 48, 35, 22, 9, 57, 44, 31, 18, 5, 53,
        40, 27, 14, 1, 49, 36, 23, 10, 58, 45, 32, 19, 6, 54, 41, 28, 15, 2, 50, 37, 24, 11, 59,
        46, 33, 20, 7, 55, 42, 29, 16, 3, 51, 38, 25, 12, 60, 47,
    ],
    [
        14, 48, 35, 22, 9, 57, 44, 31, 18, 5, 53, 40, 27, 0, 1, 49, 36, 23, 10, 58, 45, 32, 19, 6,
        54, 41, 28, 15, 2, 50, 37, 24, 11, 59, 46, 33, 20, 7, 55, 42, 29, 16, 3, 51, 38, 25, 12,
        60, 47, 34, 21, 8, 56, 43, 30, 17, 4, 52, 39, 26, 13, 61,
    ],
    [
        15, 1, 49, 36, 23, 10, 58, 45, 32, 19, 6, 54, 41, 28, 0, 2, 50, 37, 24, 11, 59, 46, 33, 20,
        7, 55, 42, 29, 16, 3, 51, 38, 25, 12, 60, 47, 34, 21, 8, 56, 43, 30, 17, 4, 52, 39, 26, 13,
        61, 48, 35, 22, 9, 57, 44, 31, 18, 5, 53, 40, 27, 14,
    ],
    [
        16, 15, 2, 50, 37, 24, 11, 59, 46, 33, 20, 7, 55, 42, 29, 0, 3, 51, 38, 25, 12, 60, 47, 34,
        21, 8, 56, 43, 30, 17, 4, 52, 39, 26, 13, 61, 48, 35, 22, 9, 57, 44, 31, 18, 5, 53, 40, 27,
        14, 1, 49, 36, 23, 10, 58, 45, 32, 19, 6, 54, 41, 28,
    ],
    [
        17, 29, 16, 3, 51, 38, 25, 12, 60, 47, 34, 21, 8, 56, 43, 30, 0, 4, 52, 39, 26, 13, 61, 48,
        35, 22, 9, 57, 44, 31, 18, 5, 53, 40, 27, 14, 1, 49, 36, 23, 10, 58, 45, 32, 19, 6, 54, 41,
        28, 15, 2, 50, 37, 24, 11, 59, 46, 33, 20, 7, 55, 42,
    ],
    [
        18, 43, 30, 17, 4, 52, 39, 26, 13, 61, 48, 35, 22, 9, 57, 44, 31, 0, 5, 53, 40, 27, 14, 1,
        49, 36, 23, 10, 58, 45, 32, 19, 6, 54, 41, 28, 15, 2, 50, 37, 24, 11, 59, 46, 33, 20, 7,
        55, 42, 29, 16, 3, 51, 38, 25, 12, 60, 47, 34, 21, 8, 56,
    ],
    [
        19, 57, 44, 31, 18, 5, 53, 40, 27, 14, 1, 49, 36, 23, 10, 58, 45, 32, 0, 6, 54, 41, 28, 15,
        2, 50, 37, 24, 11, 59, 46, 33, 20, 7, 55, 42, 29, 16, 3, 51, 38, 25, 12, 60, 47, 34, 21, 8,
        56, 43, 30, 17, 4, 52, 39, 26, 13, 61, 48, 35, 22, 9,
    ],
    [
        20, 10, 58, 45, 32, 19, 6, 54, 41, 28, 15, 2, 50, 37, 24, 11, 59, 46, 33, 0, 7, 55, 42, 29,
        16, 3, 51, 38, 25, 12, 60, 47, 34, 21, 8, 56, 43, 30, 17, 4, 52, 39, 26, 13, 61, 48, 35,
        22, 9, 57, 44, 31, 18, 5, 53, 40, 27, 14, 1, 49, 36, 23,
    ],
    [
        21, 24, 11, 59, 46, 33, 20, 7, 55, 42, 29, 16, 3, 51, 38, 25, 12, 60, 47, 34, 0, 8, 56, 43,
        30, 17, 4, 52, 39, 26, 13, 61, 48, 35, 22, 9, 57, 44, 31, 18, 5, 53, 40, 27, 14, 1, 49, 36,
        23, 10, 58, 45, 32, 19, 6, 54, 41, 28, 15, 2, 50, 37,
    ],
    [
        22, 38, 25, 12, 60, 47, 34, 21, 8, 56, 43, 30, 17, 4, 52, 39, 26, 13, 61, 48, 35, 0, 9, 57,
        44, 31, 18, 5, 53, 40, 27, 14, 1, 49, 36, 23, 10, 58, 45, 32, 19, 6, 54, 41, 28, 15, 2, 50,
        37, 24, 11, 59, 46, 33, 20, 7, 55, 42, 29, 16, 3, 51,
    ],
    [
        23, 52, 39, 26, 13, 61, 48, 35, 22, 9, 57, 44, 31, 18, 5, 53, 40, 27, 14, 1, 49, 36, 0, 10,
        58, 45, 32, 19, 6, 54, 41, 28, 15, 2, 50, 37, 24, 11, 59, 46, 33, 20, 7, 55, 42, 29, 16, 3,
        51, 38, 25, 12, 60, 47, 34, 21, 8, 56, 43, 30, 17, 4,
    ],
    [
        24, 5, 53, 40, 27, 14, 1, 49, 36, 23, 10, 58, 45, 32, 19, 6, 54, 41, 28, 15, 2, 50, 37, 0,
        11, 59, 46, 33, 20, 7, 55, 42, 29, 16, 3, 51, 38, 25, 12, 60, 47, 34, 21, 8, 56, 43, 30,
        17, 4, 52, 39, 26, 13, 61, 48, 35, 22, 9, 57, 44, 31, 18,
    ],
    [
        25, 19, 6, 54, 41, 28, 15, 2, 50, 37, 24, 11, 59, 46, 33, 20, 7, 55, 42, 29, 16, 3, 51, 38,
        0, 12, 60, 47, 34, 21, 8, 56, 43, 30, 17, 4, 52, 39, 26, 13, 61, 48, 35, 22, 9, 57, 44, 31,
        18, 5, 53, 40, 27, 14, 1, 49, 36, 23, 10, 58, 45, 32,
    ],
    [
        26, 33, 20, 7, 55, 42, 29, 16, 3, 51, 38, 25, 12, 60, 47, 34, 21, 8, 56, 43, 30, 17, 4, 52,
        39, 0, 13, 61, 48, 35, 22, 9, 57, 44, 31, 18, 5, 53, 40, 27, 14, 1, 49, 36, 23, 10, 58, 45,
        32, 19, 6, 54, 41, 28, 15, 2, 50, 37, 24, 11, 59, 46,
    ],
    [
        27, 47, 34, 21, 8, 56, 43, 30, 17, 4, 52, 39, 26, 13, 61, 48, 35, 22, 9, 57, 44, 31, 18, 5,
        53, 40, 0, 14, 1, 49, 36, 23, 10, 58, 45, 32, 19, 6, 54, 41, 28, 15, 2, 50, 37, 24, 11, 59,
        46, 33, 20, 7, 55, 42, 29, 16, 3, 51, 38, 25, 12, 60,
    ],
    [
        28, 61, 48, 35, 22, 9, 57, 44, 31, 18, 5, 53, 40, 27, 14, 1, 49, 36, 23, 10, 58, 45, 32,
        19, 6, 54, 41, 0, 15, 2, 50, 37, 24, 11, 59, 46, 33, 20, 7, 55, 42, 29, 16, 3, 51, 38, 25,
        12, 60, 47, 34, 21, 8, 56, 43, 30, 17, 4, 52, 39, 26, 13,
    ],
    [
        29, 14, 1, 49, 36, 23, 10, 58, 45, 32, 19, 6, 54, 41, 28, 15, 2, 50, 37, 24, 11, 59, 46,
        33, 20, 7, 55, 42, 0, 16, 3, 51, 38, 25, 12, 60, 47, 34, 21, 8, 56, 43, 30, 17, 4, 52, 39,
        26, 13, 61, 48, 35, 22, 9, 57, 44, 31, 18, 5, 53, 40, 27,
    ],
    [
        30, 28, 15, 2, 50, 37, 24, 11, 59, 46, 33, 20, 7, 55, 42, 29, 16, 3, 51, 38, 25, 12, 60,
        47, 34, 21, 8, 56, 43, 0, 17, 4, 52, 39, 26, 13, 61, 48, 35, 22, 9, 57, 44, 31, 18, 5, 53,
        40, 27, 14, 1, 49, 36, 23, 10, 58, 45, 32, 19, 6, 54, 41,
    ],
    [
        31, 42, 29, 16, 3, 51, 38, 25, 12, 60, 47, 34, 21, 8, 56, 43, 30, 17, 4, 52, 39, 26, 13,
        61, 48, 35, 22, 9, 57, 44, 0, 18, 5, 53, 40, 27, 14, 1, 49, 36, 23, 10, 58, 45, 32, 19, 6,
        54, 41, 28, 15, 2, 50, 37, 24, 11, 59, 46, 33, 20, 7, 55,
    ],
    [
        32, 56, 43, 30, 17, 4, 52, 39, 26, 13, 61, 48, 35, 22, 9, 57, 44, 31, 18, 5, 53, 40, 27,
        14, 1, 49, 36, 23, 10, 58, 45, 0, 19, 6, 54, 41, 28, 15, 2, 50, 37, 24, 11, 59, 46, 33, 20,
        7, 55, 42, 29, 16, 3, 51, 38, 25, 12, 60, 47, 34, 21, 8,
    ],
    [
        33, 9, 57, 44, 31, 18, 5, 53, 40, 27, 14, 1, 49, 36, 23, 10, 58, 45, 32, 19, 6, 54, 41, 28,
        15, 2, 50, 37, 24, 11, 59, 46, 0, 20, 7, 55, 42, 29, 16, 3, 51, 38, 25, 12, 60, 47, 34, 21,
        8, 56, 43, 30, 17, 4, 52, 39, 26, 13, 61, 48, 35, 22,
    ],
    [
        34, 23, 10, 58, 45, 32, 19, 6, 54, 41, 28, 15, 2, 50, 37, 24, 11, 59, 46, 33, 20, 7, 55,
        42, 29, 16, 3, 51, 38, 25, 12, 60, 47, 0, 21, 8, 56, 43, 30, 17, 4, 52, 39, 26, 13, 61, 48,
        35, 22, 9, 57, 44, 31, 18, 5, 53, 40, 27, 14, 1, 49, 36,
    ],
    [
        35, 37, 24, 11, 59, 46, 33, 20, 7, 55, 42, 29, 16, 3, 51, 38, 25, 12, 60, 47, 34, 21, 8,
        56, 43, 30, 17, 4, 52, 39, 26, 13, 61, 48, 0, 22, 9, 57, 44, 31, 18, 5, 53, 40, 27, 14, 1,
        49, 36, 23, 10, 58, 45, 32, 19, 6, 54, 41, 28, 15, 2, 50,
    ],
    [
        36, 51, 38, 25, 12, 60, 47, 34, 21, 8, 56, 43, 30, 17, 4, 52, 39, 26, 13, 61, 48, 35, 22,
        9, 57, 44, 31, 18, 5, 53, 40, 27, 14, 1, 49, 0, 23, 10, 58, 45, 32, 19, 6, 54, 41, 28, 15,
        2, 50, 37, 24, 11, 59, 46, 33, 20, 7, 55, 42, 29, 16, 3,
    ],
    [
        37, 4, 52, 39, 26, 13, 61, 48, 35, 22, 9, 57, 44, 31, 18, 5, 53, 40, 27, 14, 1, 49, 36, 23,
        10, 58, 45, 32, 19, 6, 54, 41, 28, 15, 2, 50, 0, 24, 11, 59, 46, 33, 20, 7, 55, 42, 29, 16,
        3, 51, 38, 25, 12, 60, 47, 34, 21, 8, 56, 43, 30, 17,
    ],
    [
        38, 18, 5, 53, 40, 27, 14, 1, 49, 36, 23, 10, 58, 45, 32, 19, 6, 54, 41, 28, 15, 2, 50, 37,
        24, 11, 59, 46, 33, 20, 7, 55, 42, 29, 16, 3, 51, 0, 25, 12, 60, 47, 34, 21, 8, 56, 43, 30,
        17, 4, 52, 39, 26, 13, 61, 48, 35, 22, 9, 57, 44, 31,
    ],
    [
        39, 32, 19, 6, 54, 41, 28, 15, 2, 50, 37, 24, 11, 59, 46, 33, 20, 7, 55, 42, 29, 16, 3, 51,
        38, 25, 12, 60, 47, 34, 21, 8, 56, 43, 30, 17, 4, 52, 0, 26, 13, 61, 48, 35, 22, 9, 57, 44,
        31, 18, 5, 53, 40, 27, 14, 1, 49, 36, 23, 10, 58, 45,
    ],
    [
        40, 46, 33, 20, 7, 55, 42, 29, 16, 3, 51, 38, 25, 12, 60, 47, 34, 21, 8, 56, 43, 30, 17, 4,
        52, 39, 26, 13, 61, 48, 35, 22, 9, 57, 44, 31, 18, 5, 53, 0, 27, 14, 1, 49, 36, 23, 10, 58,
        45, 32, 19, 6, 54, 41, 28, 15, 2, 50, 37, 24, 11, 59,
    ],
    [
        41, 60, 47, 34, 21, 8, 56, 43, 30, 17, 4, 52, 39, 26, 13, 61, 48, 35, 22, 9, 57, 44, 31,
        18, 5, 53, 40, 27, 14, 1, 49, 36, 23, 10, 58, 45, 32, 19, 6, 54, 0, 28, 15, 2, 50, 37, 24,
        11, 59, 46, 33, 20, 7, 55, 42, 29, 16, 3, 51, 38, 25, 12,
    ],
    [
        42, 13, 61, 48, 35, 22, 9, 57, 44, 31, 18, 5, 53, 40, 27, 14, 1, 49, 36, 23, 10, 58, 45,
        32, 19, 6, 54, 41, 28, 15, 2, 50, 37, 24, 11, 59, 46, 33, 20, 7, 55, 0, 29, 16, 3, 51, 38,
        25, 12, 60, 47, 34, 21, 8, 56, 43, 30, 17, 4, 52, 39, 26,
    ],
    [
        43, 27, 14, 1, 49, 36, 23, 10, 58, 45, 32, 19, 6, 54, 41, 28, 15, 2, 50, 37, 24, 11, 59,
        46, 33, 20, 7, 55, 42, 29, 16, 3, 51, 38, 25, 12, 60, 47, 34, 21, 8, 56, 0, 30, 17, 4, 52,
        39, 26, 13, 61, 48, 35, 22, 9, 57, 44, 31, 18, 5, 53, 40,
    ],
    [
        44, 41, 28, 15, 2, 50, 37, 24, 11, 59, 46, 33, 20, 7, 55, 42, 29, 16, 3, 51, 38, 25, 12,
        60, 47, 34, 21, 8, 56, 43, 30, 17, 4, 52, 39, 26, 13, 61, 48, 35, 22, 9, 57, 0, 31, 18, 5,
        53, 40, 27, 14, 1, 49, 36, 23, 10, 58, 45, 32, 19, 6, 54,
    ],
    [
        45, 55, 42, 29, 16, 3, 51, 38, 25, 12, 60, 47, 34, 21, 8, 56, 43, 30, 17, 4, 52, 39, 26,
        13, 61, 48, 35, 22, 9, 57, 44, 31, 18, 5, 53, 40, 27, 14, 1, 49, 36, 23, 10, 58, 0, 32, 19,
        6, 54, 41, 28, 15, 2, 50, 37, 24, 11, 59, 46, 33, 20, 7,
    ],
    [
        46, 8, 56, 43, 30, 17, 4, 52, 39, 26, 13, 61, 48, 35, 22, 9, 57, 44, 31, 18, 5, 53, 40, 27,
        14, 1, 49, 36, 23, 10, 58, 45, 32, 19, 6, 54, 41, 28, 15, 2, 50, 37, 24, 11, 59, 0, 33, 20,
        7, 55, 42, 29, 16, 3, 51, 38, 25, 12, 60, 47, 34, 21,
    ],
    [
        47, 22, 9, 57, 44, 31, 18, 5, 53, 40, 27, 14, 1, 49, 36, 23, 10, 58, 45, 32, 19, 6, 54, 41,
        28, 15, 2, 50, 37, 24, 11, 59, 46, 33, 20, 7, 55, 42, 29, 16, 3, 51, 38, 25, 12, 60, 0, 34,
        21, 8, 56, 43, 30, 17, 4, 52, 39, 26, 13, 61, 48, 35,
    ],
    [
        48, 36, 23, 10, 58, 45, 32, 19, 6, 54, 41, 28, 15, 2, 50, 37, 24, 11, 59, 46, 33, 20, 7,
        55, 42, 29, 16, 3, 51, 38, 25, 12, 60, 47, 34, 21, 8, 56, 43, 30, 17, 4, 52, 39, 26, 13,
        61, 0, 35, 22, 9, 57, 44, 31, 18, 5, 53, 40, 27, 14, 1, 49,
    ],
    [
        49, 50, 37, 24, 11, 59, 46, 33, 20, 7, 55, 42, 29, 16, 3, 51, 38, 25, 12, 60, 47, 34, 21,
        8, 56, 43, 30, 17, 4, 52, 39, 26, 13, 61, 48, 35, 22, 9, 57, 44, 31, 18, 5, 53, 40, 27, 14,
        1, 0, 36, 23, 10, 58, 45, 32, 19, 6, 54, 41, 28, 15, 2,
    ],
    [
        50, 3, 51, 38, 25, 12, 60, 47, 34, 21, 8, 56, 43, 30, 17, 4, 52, 39, 26, 13, 61, 48, 35,
        22, 9, 57, 44, 31, 18, 5, 53, 40, 27, 14, 1, 49, 36, 23, 10, 58, 45, 32, 19, 6, 54, 41, 28,
        15, 2, 0, 37, 24, 11, 59, 46, 33, 20, 7, 55, 42, 29, 16,
    ],
    [
        51, 17, 4, 52, 39, 26, 13, 61, 48, 35, 22, 9, 57, 44, 31, 18, 5, 53, 40, 27, 14, 1, 49, 36,
        23, 10, 58, 45, 32, 19, 6, 54, 41, 28, 15, 2, 50, 37, 24, 11, 59, 46, 33, 20, 7, 55, 42,
        29, 16, 3, 0, 38, 25, 12, 60, 47, 34, 21, 8, 56, 43, 30,
    ],
    [
        52, 31, 18, 5, 53, 40, 27, 14, 1, 49, 36, 23, 10, 58, 45, 32, 19, 6, 54, 41, 28, 15, 2, 50,
        37, 24, 11, 59, 46, 33, 20, 7, 55, 42, 29, 16, 3, 51, 38, 25, 12, 60, 47, 34, 21, 8, 56,
        43, 30, 17, 4, 0, 39, 26, 13, 61, 48, 35, 22, 9, 57, 44,
    ],
    [
        53, 45, 32, 19, 6, 54, 41, 28, 15, 2, 50, 37, 24, 11, 59, 46, 33, 20, 7, 55, 42, 29, 16, 3,
        51, 38, 25, 12, 60, 47, 34, 21, 8, 56, 43, 30, 17, 4, 52, 39, 26, 13, 61, 48, 35, 22, 9,
        57, 44, 31, 18, 5, 0, 40, 27, 14, 1, 49, 36, 23, 10, 58,
    ],
    [
        54, 59, 46, 33, 20, 7, 55, 42, 29, 16, 3, 51, 38, 25, 12, 60, 47, 34, 21, 8, 56, 43, 30,
        17, 4, 52, 39, 26, 13, 61, 48, 35, 22, 9, 57, 44, 31, 18, 5, 53, 40, 27, 14, 1, 49, 36, 23,
        10, 58, 45, 32, 19, 6, 0, 41, 28, 15, 2, 50, 37, 24, 11,
    ],
    [
        55, 12, 60, 47, 34, 21, 8, 56, 43, 30, 17, 4, 52, 39, 26, 13, 61, 48, 35, 22, 9, 57, 44,
        31, 18, 5, 53, 40, 27, 14, 1, 49, 36, 23, 10, 58, 45, 32, 19, 6, 54, 41, 28, 15, 2, 50, 37,
        24, 11, 59, 46, 33, 20, 7, 0, 42, 29, 16, 3, 51, 38, 25,
    ],
    [
        56, 26, 13, 61, 48, 35, 22, 9, 57, 44, 31, 18, 5, 53, 40, 27, 14, 1, 49, 36, 23, 10, 58,
        45, 32, 19, 6, 54, 41, 28, 15, 2, 50, 37, 24, 11, 59, 46, 33, 20, 7, 55, 42, 29, 16, 3, 51,
        38, 25, 12, 60, 47, 34, 21, 8, 0, 43, 30, 17, 4, 52, 39,
    ],
    [
        57, 40, 27, 14, 1, 49, 36, 23, 10, 58, 45, 32, 19, 6, 54, 41, 28, 15, 2, 50, 37, 24, 11,
        59, 46, 33, 20, 7, 55, 42, 29, 16, 3, 51, 38, 25, 12, 60, 47, 34, 21, 8, 56, 43, 30, 17, 4,
        52, 39, 26, 13, 61, 48, 35, 22, 9, 0, 44, 31, 18, 5, 53,
    ],
    [
        58, 54, 41, 28, 15, 2, 50, 37, 24, 11, 59, 46, 33, 20, 7, 55, 42, 29, 16, 3, 51, 38, 25,
        12, 60, 47, 34, 21, 8, 56, 43, 30, 17, 4, 52, 39, 26, 13, 61, 48, 35, 22, 9, 57, 44, 31,
        18, 5, 53, 40, 27, 14, 1, 49, 36, 23, 10, 0, 45, 32, 19, 6,
    ],
    [
        59, 7, 55, 42, 29, 16, 3, 51, 38, 25, 12, 60, 47, 34, 21, 8, 56, 43, 30, 17, 4, 52, 39, 26,
        13, 61, 48, 35, 22, 9, 57, 44, 31, 18, 5, 53, 40, 27, 14, 1, 49, 36, 23, 10, 58, 45, 32,
        19, 6, 54, 41, 28, 15, 2, 50, 37, 24, 11, 0, 46, 33, 20,
    ],
    [
        60, 21, 8, 56, 43, 30, 17, 4, 52, 39, 26, 13, 61, 48, 35, 22, 9, 57, 44, 31, 18, 5, 53, 40,
        27, 14, 1, 49, 36, 23, 10, 58, 45, 32, 19, 6, 54, 41, 28, 15, 2, 50, 37, 24, 11, 59, 46,
        33, 20, 7, 55, 42, 29, 16, 3, 51, 38, 25, 12, 0, 47, 34,
    ],
    [
        61, 35, 22, 9, 57, 44, 31, 18, 5, 53, 40, 27, 14, 1, 49, 36, 23, 10, 58, 45, 32, 19, 6, 54,
        41, 28, 15, 2, 50, 37, 24, 11, 59, 46, 33, 20, 7, 55, 42, 29, 16, 3, 51, 38, 25, 12, 60,
        47, 34, 21, 8, 56, 43, 30, 17, 4, 52, 39, 26, 13, 0, 48,
    ],
    [
        1, 49, 36, 23, 10, 58, 45, 32, 19, 6, 54, 41, 28, 15, 2, 50, 37, 24, 11, 59, 46, 33, 20, 7,
        55, 42, 29, 16, 3, 51, 38, 25, 12, 60, 47, 34, 21, 8, 56, 43, 30, 17, 4, 52, 39, 26, 13,
        61, 48, 35, 22, 9, 57, 44, 31, 18, 5, 53, 40, 27, 14, 0,
    ],
];

/// Ordnung 64: x*y:=2x+y mit Rechnung in GF(2^6)
const BASE64: [[u8; 64]; 64] = [
    [
        0, 2, 4, 6, 8, 10, 12, 14, 16, 18, 20, 22, 24, 26, 28, 30, 32, 34, 36, 38, 40, 42, 44, 46,
        48, 50, 52, 54, 56, 58, 60, 62, 3, 1, 7, 5, 11, 9, 15, 13, 19, 17, 23, 21, 27, 25, 31, 29,
        35, 33, 39, 37, 43, 41, 47, 45, 51, 49, 55, 53, 59, 57, 63, 61,
    ],
    [
        2, 0, 6, 4, 10, 8, 14, 12, 18, 16, 22, 20, 26, 24, 30, 28, 34, 32, 38, 36, 42, 40, 46, 44,
        50, 48, 54, 52, 58, 56, 62, 60, 1, 3, 5, 7, 9, 11, 13, 15, 17, 19, 21, 23, 25, 27, 29, 31,
        33, 35, 37, 39, 41, 43, 45, 47, 49, 51, 53, 55, 57, 59, 61, 63,
    ],
    [
        4, 6, 0, 2, 12, 14, 8, 10, 20, 22, 16, 18, 28, 30, 24, 26, 36, 38, 32, 34, 44, 46, 40, 42,
        52, 54, 48, 50, 60, 62, 56, 58, 7, 5, 3, 1, 15, 13, 11, 9, 23, 21, 19, 17, 31, 29, 27, 25,
        39, 37, 35, 33, 47, 45, 43, 41, 55, 53, 51, 49, 63, 61, 59, 57,
    ],
    [
        6, 4, 2, 0, 14, 12, 10, 8, 22, 20, 18, 16, 30, 28, 26, 24, 38, 36, 34, 32, 46, 44, 42, 40,
        54, 52, 50, 48, 62, 60, 58, 56, 5, 7, 1, 3, 13, 15, 9, 11, 21, 23, 17, 19, 29, 31, 25, 27,
        37, 39, 33, 35, 45, 47, 41, 43, 53, 55, 49, 51, 61, 63, 57, 59,
    ],
    [
        8, 10, 12, 14, 0, 2, 4, 6, 24, 26, 28, 30, 16, 18, 20, 22, 40, 42, 44, 46, 32, 34, 36, 38,
        56, 58, 60, 62, 48, 50, 52, 54, 11, 9, 15, 13, 3, 1, 7, 5, 27, 25, 31, 29, 19, 17, 23, 21,
        43, 41, 47, 45, 35, 33, 39, 37, 59, 57, 63, 61, 51, 49, 55, 53,
    ],
    [
        10, 8, 14, 12, 2, 0, 6, 4, 26, 24, 30, 28, 18, 16, 22, 20, 42, 40, 46, 44, 34, 32, 38, 36,
        58, 56, 62, 60, 50, 48, 54, 52, 9, 11, 13, 15, 1, 3, 5, 7, 25, 27, 29, 31, 17, 19, 21, 23,
        41, 43, 45, 47, 33, 35, 37, 39, 57, 59, 61, 63, 49, 51, 53, 55,
    ],
    [
        12, 14, 8, 10, 4, 6, 0, 2, 28, 30, 24, 26, 20, 22, 16, 18, 44, 46, 40, 42, 36, 38, 32, 34,
        60, 62, 56, 58, 52, 54, 48, 50, 15, 13, 11, 9, 7, 5, 3, 1, 31, 29, 27, 25, 23, 21, 19, 17,
        47, 45, 43, 41, 39, 37, 35, 33, 63, 61, 59, 57, 55, 53, 51, 49,
    ],
    [
        14, 12, 10, 8, 6, 4, 2, 0, 30, 28, 26, 24, 22, 20, 18, 16, 46, 44, 42, 40, 38, 36, 34, 32,
        62, 60, 58, 56, 54, 52, 50, 48, 13, 15, 9, 11, 5, 7, 1, 3, 29, 31, 25, 27, 21, 23, 17, 19,
        45, 47, 41, 43, 37, 39, 33, 35, 61, 63, 57, 59, 53, 55, 49, 51,
    ],
    [
        16, 18, 20, 22, 24, 26, 28, 30, 0, 2, 4, 6, 8, 10, 12, 14, 48, 50, 52, 54, 56, 58, 60, 62,
        32, 34, 36, 38, 40, 42, 44, 46, 19, 17, 23, 21, 27, 25, 31, 29, 3, 1, 7, 5, 11, 9, 15, 13,
        51, 49, 55, 53, 59, 57, 63, 61, 35, 33, 39, 37, 43, 41, 47, 45,
    ],
    [
        18, 16, 22, 20, 26, 24, 30, 28, 2, 0, 6, 4, 10, 8, 14, 12, 50, 48, 54, 52, 58, 56, 62, 60,
        34, 32, 38, 36, 42, 40, 46, 44, 17, 19, 21, 23, 25, 27, 29, 31, 1, 3, 5, 7, 9, 11, 13, 15,
        49, 51, 53, 55, 57, 59, 61, 63, 33, 35, 37, 39, 41, 43, 45, 47,
    ],
    [
        20, 22, 16, 18, 28, 30, 24, 26, 4, 6, 0, 2, 12, 14, 8, 10, 52, 54, 48, 50, 60, 62, 56, 58,
        36, 38, 32, 34, 44, 46, 40, 42, 23, 21, 19, 17, 31, 29, 27, 25, 7, 5, 3, 1, 15, 13, 11, 9,
        55, 53, 51, 49, 63, 61, 59, 57, 39, 37, 35, 33, 47, 45, 43, 41,
    ],
    [
        22, 20, 18, 16, 30, 28, 26, 24, 6, 4, 2, 0, 14, 12, 10, 8, 54, 52, 50, 48, 62, 60, 58, 56,
        38, 36, 34, 32, 46, 44, 42, 40, 21, 23, 17, 19, 29, 31, 25, 27, 5, 7, 1, 3, 13, 15, 9, 11,
        53, 55, 49, 51, 61, 63, 57, 59, 37, 39, 33, 35, 45, 47, 41, 43,
    ],
    [
        24, 26, 28, 30, 16, 18, 20, 22, 8, 10, 12, 14, 0, 2, 4, 6, 56, 58, 60, 62, 48, 50, 52, 54,
        40, 42, 44, 46, 32, 34, 36, 38, 27, 25, 31, 29, 19, 17, 23, 21, 11, 9, 15, 13, 3, 1, 7, 5,
        59, 57, 63, 61, 51, 49, 55, 53, 43, 41, 47, 45, 35, 33, 39, 37,
    ],
    [
        26, 24, 30, 28, 18, 16, 22, 20, 10, 8, 14, 12, 2, 0, 6, 4, 58, 56, 62, 60, 50, 48, 54, 52,
        42, 40, 46, 44, 34, 32, 38, 36, 25, 27, 29, 31, 17, 19, 21, 23, 9, 11, 13, 15, 1, 3, 5, 7,
        57, 59, 61, 63, 49, 51, 53, 55, 41, 43, 45, 47, 33, 35, 37, 39,
    ],
    [
        28, 30, 24, 26, 20, 22, 16, 18, 12, 14, 8, 10, 4, 6, 0, 2, 60, 62, 56, 58, 52, 54, 48, 50,
        44, 46, 40, 42, 36, 38, 32, 34, 31, 29, 27, 25, 23, 21, 19, 17, 15, 13, 11, 9, 7, 5, 3, 1,
        63, 61, 59, 57, 55, 53, 51, 49, 47, 45, 43, 41, 39, 37, 35, 33,
    ],
    [
        30, 28, 26, 24, 22, 20, 18, 16, 14, 12, 10, 8, 6, 4, 2, 0, 62, 60, 58, 56, 54, 52, 50, 48,
        46, 44, 42, 40, 38, 36, 34, 32, 29, 31, 25, 27, 21, 23, 17, 19, 13, 15, 9, 11, 5, 7, 1, 3,
        61, 63, 57, 59, 53, 55, 49, 51, 45, 47, 41, 43, 37, 39, 33, 35,
    ],
    [
        32, 34, 36, 38, 40, 42, 44, 46, 48, 50, 52, 54, 56, 58, 60, 62, 0, 2, 4, 6, 8, 10, 12, 14,
        16, 18, 20, 22, 24, 26, 28, 30, 35, 33, 39, 37, 43, 41, 47, 45, 51, 49, 55, 53, 59, 57, 63,
        61, 3, 1, 7, 5, 11, 9, 15, 13, 19, 17, 23, 21, 27, 25, 31, 29,
    ],
    [
        34, 32, 38, 36, 42, 40, 46, 44, 50, 48, 54, 52, 58, 56, 62, 60, 2, 0, 6, 4, 10, 8, 14, 12,
        18, 16, 22, 20, 26, 24, 30, 28, 33, 35, 37, 39, 41, 43, 45, 47, 49, 51, 53, 55, 57, 59, 61,
        63, 1, 3, 5, 7, 9, 11, 13, 15, 17, 19, 21, 23, 25, 27, 29, 31,
    ],
    [
        36, 38, 32, 34, 44, 46, 40, 42, 52, 54, 48, 50, 60, 62, 56, 58, 4, 6, 0, 2, 12, 14, 8, 10,
        20, 22, 16, 18, 28, 30, 24, 26, 39, 37, 35, 33, 47, 45, 43, 41, 55, 53, 51, 49, 63, 61, 59,
        57, 7, 5, 3, 1, 15, 13, 11, 9, 23, 21, 19, 17, 31, 29, 27, 25,
    ],
    [
        38, 36, 34, 32, 46, 44, 42, 40, 54, 52, 50, 48, 62, 60, 58, 56, 6, 4, 2, 0, 14, 12, 10, 8,
        22, 20, 18, 16, 30, 28, 26, 24, 37, 39, 33, 35, 45, 47, 41, 43, 53, 55, 49, 51, 61, 63, 57,
        59, 5, 7, 1, 3, 13, 15, 9, 11, 21, 23, 17, 19, 29, 31, 25, 27,
    ],
    [
        40, 42, 44, 46, 32, 34, 36, 38, 56, 58, 60, 62, 48, 50, 52, 54, 8, 10, 12, 14, 0, 2, 4, 6,
        24, 26, 28, 30, 16, 18, 20, 22, 43, 41, 47, 45, 35, 33, 39, 37, 59, 57, 63, 61, 51, 49, 55,
        53, 11, 9, 15, 13, 3, 1, 7, 5, 27, 25, 31, 29, 19, 17, 23, 21,
    ],
    [
        42, 40, 46, 44, 34, 32, 38, 36, 58, 56, 62, 60, 50, 48, 54, 52, 10, 8, 14, 12, 2, 0, 6, 4,
        26, 24, 30, 28, 18, 16, 22, 20, 41, 43, 45, 47, 33, 35, 37, 39, 57, 59, 61, 63, 49, 51, 53,
        55, 9, 11, 13, 15, 1, 3, 5, 7, 25, 27, 29, 31, 17, 19, 21, 23,
    ],
    [
        44, 46, 40, 42, 36, 38, 32, 34, 60, 62, 56, 58, 52, 54, 48, 50, 12, 14, 8, 10, 4, 6, 0, 2,
        28, 30, 24, 26, 20, 22, 16, 18, 47, 45, 43, 41, 39, 37, 35, 33, 63, 61, 59, 57, 55, 53, 51,
        49, 15, 13, 11, 9, 7, 5, 3, 1, 31, 29, 27, 25, 23, 21, 19, 17,
    ],
    [
        46, 44, 42, 40, 38, 36, 34, 32, 62, 60, 58, 56, 54, 52, 50, 48, 14, 12, 10, 8, 6, 4, 2, 0,
        30, 28, 26, 24, 22, 20, 18, 16, 45, 47, 41, 43, 37, 39, 33, 35, 61, 63, 57, 59, 53, 55, 49,
        51, 13, 15, 9, 11, 5, 7, 1, 3, 29, 31, 25, 27, 21, 23, 17, 19,
    ],
    [
        48, 50, 52, 54, 56, 58, 60, 62, 32, 34, 36, 38, 40, 42, 44, 46, 16, 18, 20, 22, 24, 26, 28,
        30, 0, 2, 4, 6, 8, 10, 12, 14, 51, 49, 55, 53, 59, 57, 63, 61, 35, 33, 39, 37, 43, 41, 47,
        45, 19, 17, 23, 21, 27, 25, 31, 29, 3, 1, 7, 5, 11, 9, 15, 13,
    ],
    [
        50, 48, 54, 52, 58, 56, 62, 60, 34, 32, 38, 36, 42, 40, 46, 44, 18, 16, 22, 20, 26, 24, 30,
        28, 2, 0, 6, 4, 10, 8, 14, 12, 49, 51, 53, 55, 57, 59, 61, 63, 33, 35, 37, 39, 41, 43, 45,
        47, 17, 19, 21, 23, 25, 27, 29, 31, 1, 3, 5, 7, 9, 11, 13, 15,
    ],
    [
        52, 54, 48, 50, 60, 62, 56, 58, 36, 38, 32, 34, 44, 46, 40, 42, 20, 22, 16, 18, 28, 30, 24,
        26, 4, 6, 0, 2, 12, 14, 8, 10, 55, 53, 51, 49, 63, 61, 59, 57, 39, 37, 35, 33, 47, 45, 43,
        41, 23, 21, 19, 17, 31, 29, 27, 25, 7, 5, 3, 1, 15, 13, 11, 9,
    ],
    [
        54, 52, 50, 48, 62, 60, 58, 56, 38, 36, 34, 32, 46, 44, 42, 40, 22, 20, 18, 16, 30, 28, 26,
        24, 6, 4, 2, 0, 14, 12, 10, 8, 53, 55, 49, 51, 61, 63, 57, 59, 37, 39, 33, 35, 45, 47, 41,
        43, 21, 23, 17, 19, 29, 31, 25, 27, 5, 7, 1, 3, 13, 15, 9, 11,
    ],
    [
        56, 58, 60, 62, 48, 50, 52, 54, 40, 42, 44, 46, 32, 34, 36, 38, 24, 26, 28, 30, 16, 18, 20,
        22, 8, 10, 12, 14, 0, 2, 4, 6, 59, 57, 63, 61, 51, 49, 55, 53, 43, 41, 47, 45, 35, 33, 39,
        37, 27, 25, 31, 29, 19, 17, 23, 21, 11, 9, 15, 13, 3, 1, 7, 5,
    ],
    [
        58, 56, 62, 60, 50, 48, 54, 52, 42, 40, 46, 44, 34, 32, 38, 36, 26, 24, 30, 28, 18, 16, 22,
        20, 10, 8, 14, 12, 2, 0, 6, 4, 57, 59, 61, 63, 49, 51, 53, 55, 41, 43, 45, 47, 33, 35, 37,
        39, 25, 27, 29, 31, 17, 19, 21, 23, 9, 11, 13, 15, 1, 3, 5, 7,
    ],
    [
        60, 62, 56, 58, 52, 54, 48, 50, 44, 46, 40, 42, 36, 38, 32, 34, 28, 30, 24, 26, 20, 22, 16,
        18, 12, 14, 8, 10, 4, 6, 0, 2, 63, 61, 59, 57, 55, 53, 51, 49, 47, 45, 43, 41, 39, 37, 35,
        33, 31, 29, 27, 25, 23, 21, 19, 17, 15, 13, 11, 9, 7, 5, 3, 1,
    ],
    [
        62, 60, 58, 56, 54, 52, 50, 48, 46, 44, 42, 40, 38, 36, 34, 32, 30, 28, 26, 24, 22, 20, 18,
        16, 14, 12, 10, 8, 6, 4, 2, 0, 61, 63, 57, 59, 53, 55, 49, 51, 45, 47, 41, 43, 37, 39, 33,
        35, 29, 31, 25, 27, 21, 23, 17, 19, 13, 15, 9, 11, 5, 7, 1, 3,
    ],
    [
        3, 1, 7, 5, 11, 9, 15, 13, 19, 17, 23, 21, 27, 25, 31, 29, 35, 33, 39, 37, 43, 41, 47, 45,
        51, 49, 55, 53, 59, 57, 63, 61, 0, 2, 4, 6, 8, 10, 12, 14, 16, 18, 20, 22, 24, 26, 28, 30,
        32, 34, 36, 38, 40, 42, 44, 46, 48, 50, 52, 54, 56, 58, 60, 62,
    ],
    [
        1, 3, 5, 7, 9, 11, 13, 15, 17, 19, 21, 23, 25, 27, 29, 31, 33, 35, 37, 39, 41, 43, 45, 47,
        49, 51, 53, 55, 57, 59, 61, 63, 2, 0, 6, 4, 10, 8, 14, 12, 18, 16, 22, 20, 26, 24, 30, 28,
        34, 32, 38, 36, 42, 40, 46, 44, 50, 48, 54, 52, 58, 56, 62, 60,
    ],
    [
        7, 5, 3, 1, 15, 13, 11, 9, 23, 21, 19, 17, 31, 29, 27, 25, 39, 37, 35, 33, 47, 45, 43, 41,
        55, 53, 51, 49, 63, 61, 59, 57, 4, 6, 0, 2, 12, 14, 8, 10, 20, 22, 16, 18, 28, 30, 24, 26,
        36, 38, 32, 34, 44, 46, 40, 42, 52, 54, 48, 50, 60, 62, 56, 58,
    ],
    [
        5, 7, 1, 3, 13, 15, 9, 11, 21, 23, 17, 19, 29, 31, 25, 27, 37, 39, 33, 35, 45, 47, 41, 43,
        53, 55, 49, 51, 61, 63, 57, 59, 6, 4, 2, 0, 14, 12, 10, 8, 22, 20, 18, 16, 30, 28, 26, 24,
        38, 36, 34, 32, 46, 44, 42, 40, 54, 52, 50, 48, 62, 60, 58, 56,
    ],
    [
        11, 9, 15, 13, 3, 1, 7, 5, 27, 25, 31, 29, 19, 17, 23, 21, 43, 41, 47, 45, 35, 33, 39, 37,
        59, 57, 63, 61, 51, 49, 55, 53, 8, 10, 12, 14, 0, 2, 4, 6, 24, 26, 28, 30, 16, 18, 20, 22,
        40, 42, 44, 46, 32, 34, 36, 38, 56, 58, 60, 62, 48, 50, 52, 54,
    ],
    [
        9, 11, 13, 15, 1, 3, 5, 7, 25, 27, 29, 31, 17, 19, 21, 23, 41, 43, 45, 47, 33, 35, 37, 39,
        57, 59, 61, 63, 49, 51, 53, 55, 10, 8, 14, 12, 2, 0, 6, 4, 26, 24, 30, 28, 18, 16, 22, 20,
        42, 40, 46, 44, 34, 32, 38, 36, 58, 56, 62, 60, 50, 48, 54, 52,
    ],
    [
        15, 13, 11, 9, 7, 5, 3, 1, 31, 29, 27, 25, 23, 21, 19, 17, 47, 45, 43, 41, 39, 37, 35, 33,
        63, 61, 59, 57, 55, 53, 51, 49, 12, 14, 8, 10, 4, 6, 0, 2, 28, 30, 24, 26, 20, 22, 16, 18,
        44, 46, 40, 42, 36, 38, 32, 34, 60, 62, 56, 58, 52, 54, 48, 50,
    ],
    [
        13, 15, 9, 11, 5, 7, 1, 3, 29, 31, 25, 27, 21, 23, 17, 19, 45, 47, 41, 43, 37, 39, 33, 35,
        61, 63, 57, 59, 53, 55, 49, 51, 14, 12, 10, 8, 6, 4, 2, 0, 30, 28, 26, 24, 22, 20, 18, 16,
        46, 44, 42, 40, 38, 36, 34, 32, 62, 60, 58, 56, 54, 52, 50, 48,
    ],
    [
        19, 17, 23, 21, 27, 25, 31, 29, 3, 1, 7, 5, 11, 9, 15, 13, 51, 49, 55, 53, 59, 57, 63, 61,
        35, 33, 39, 37, 43, 41, 47, 45, 16, 18, 20, 22, 24, 26, 28, 30, 0, 2, 4, 6, 8, 10, 12, 14,
        48, 50, 52, 54, 56, 58, 60, 62, 32, 34, 36, 38, 40, 42, 44, 46,
    ],
    [
        17, 19, 21, 23, 25, 27, 29, 31, 1, 3, 5, 7, 9, 11, 13, 15, 49, 51, 53, 55, 57, 59, 61, 63,
        33, 35, 37, 39, 41, 43, 45, 47, 18, 16, 22, 20, 26, 24, 30, 28, 2, 0, 6, 4, 10, 8, 14, 12,
        50, 48, 54, 52, 58, 56, 62, 60, 34, 32, 38, 36, 42, 40, 46, 44,
    ],
    [
        23, 21, 19, 17, 31, 29, 27, 25, 7, 5, 3, 1, 15, 13, 11, 9, 55, 53, 51, 49, 63, 61, 59, 57,
        39, 37, 35, 33, 47, 45, 43, 41, 20, 22, 16, 18, 28, 30, 24, 26, 4, 6, 0, 2, 12, 14, 8, 10,
        52, 54, 48, 50, 60, 62, 56, 58, 36, 38, 32, 34, 44, 46, 40, 42,
    ],
    [
        21, 23, 17, 19, 29, 31, 25, 27, 5, 7, 1, 3, 13, 15, 9, 11, 53, 55, 49, 51, 61, 63, 57, 59,
        37, 39, 33, 35, 45, 47, 41, 43, 22, 20, 18, 16, 30, 28, 26, 24, 6, 4, 2, 0, 14, 12, 10, 8,
        54, 52, 50, 48, 62, 60, 58, 56, 38, 36, 34, 32, 46, 44, 42, 40,
    ],
    [
        27, 25, 31, 29, 19, 17, 23, 21, 11, 9, 15, 13, 3, 1, 7, 5, 59, 57, 63, 61, 51, 49, 55, 53,
        43, 41, 47, 45, 35, 33, 39, 37, 24, 26, 28, 30, 16, 18, 20, 22, 8, 10, 12, 14, 0, 2, 4, 6,
        56, 58, 60, 62, 48, 50, 52, 54, 40, 42, 44, 46, 32, 34, 36, 38,
    ],
    [
        25, 27, 29, 31, 17, 19, 21, 23, 9, 11, 13, 15, 1, 3, 5, 7, 57, 59, 61, 63, 49, 51, 53, 55,
        41, 43, 45, 47, 33, 35, 37, 39, 26, 24, 30, 28, 18, 16, 22, 20, 10, 8, 14, 12, 2, 0, 6, 4,
        58, 56, 62, 60, 50, 48, 54, 52, 42, 40, 46, 44, 34, 32, 38, 36,
    ],
    [
        31, 29, 27, 25, 23, 21, 19, 17, 15, 13, 11, 9, 7, 5, 3, 1, 63, 61, 59, 57, 55, 53, 51, 49,
        47, 45, 43, 41, 39, 37, 35, 33, 28, 30, 24, 26, 20, 22, 16, 18, 12, 14, 8, 10, 4, 6, 0, 2,
        60, 62, 56, 58, 52, 54, 48, 50, 44, 46, 40, 42, 36, 38, 32, 34,
    ],
    [
        29, 31, 25, 27, 21, 23, 17, 19, 13, 15, 9, 11, 5, 7, 1, 3, 61, 63, 57, 59, 53, 55, 49, 51,
        45, 47, 41, 43, 37, 39, 33, 35, 30, 28, 26, 24, 22, 20, 18, 16, 14, 12, 10, 8, 6, 4, 2, 0,
        62, 60, 58, 56, 54, 52, 50, 48, 46, 44, 42, 40, 38, 36, 34, 32,
    ],
    [
        35, 33, 39, 37, 43, 41, 47, 45, 51, 49, 55, 53, 59, 57, 63, 61, 3, 1, 7, 5, 11, 9, 15, 13,
        19, 17, 23, 21, 27, 25, 31, 29, 32, 34, 36, 38, 40, 42, 44, 46, 48, 50, 52, 54, 56, 58, 60,
        62, 0, 2, 4, 6, 8, 10, 12, 14, 16, 18, 20, 22, 24, 26, 28, 30,
    ],
    [
        33, 35, 37, 39, 41, 43, 45, 47, 49, 51, 53, 55, 57, 59, 61, 63, 1, 3, 5, 7, 9, 11, 13, 15,
        17, 19, 21, 23, 25, 27, 29, 31, 34, 32, 38, 36, 42, 40, 46, 44, 50, 48, 54, 52, 58, 56, 62,
        60, 2, 0, 6, 4, 10, 8, 14, 12, 18, 16, 22, 20, 26, 24, 30, 28,
    ],
    [
        39, 37, 35, 33, 47, 45, 43, 41, 55, 53, 51, 49, 63, 61, 59, 57, 7, 5, 3, 1, 15, 13, 11, 9,
        23, 21, 19, 17, 31, 29, 27, 25, 36, 38, 32, 34, 44, 46, 40, 42, 52, 54, 48, 50, 60, 62, 56,
        58, 4, 6, 0, 2, 12, 14, 8, 10, 20, 22, 16, 18, 28, 30, 24, 26,
    ],
    [
        37, 39, 33, 35, 45, 47, 41, 43, 53, 55, 49, 51, 61, 63, 57, 59, 5, 7, 1, 3, 13, 15, 9, 11,
        21, 23, 17, 19, 29, 31, 25, 27, 38, 36, 34, 32, 46, 44, 42, 40, 54, 52, 50, 48, 62, 60, 58,
        56, 6, 4, 2, 0, 14, 12, 10, 8, 22, 20, 18, 16, 30, 28, 26, 24,
    ],
    [
        43, 41, 47, 45, 35, 33, 39, 37, 59, 57, 63, 61, 51, 49, 55, 53, 11, 9, 15, 13, 3, 1, 7, 5,
        27, 25, 31, 29, 19, 17, 23, 21, 40, 42, 44, 46, 32, 34, 36, 38, 56, 58, 60, 62, 48, 50, 52,
        54, 8, 10, 12, 14, 0, 2, 4, 6, 24, 26, 28, 30, 16, 18, 20, 22,
    ],
    [
        41, 43, 45, 47, 33, 35, 37, 39, 57, 59, 61, 63, 49, 51, 53, 55, 9, 11, 13, 15, 1, 3, 5, 7,
        25, 27, 29, 31, 17, 19, 21, 23, 42, 40, 46, 44, 34, 32, 38, 36, 58, 56, 62, 60, 50, 48, 54,
        52, 10, 8, 14, 12, 2, 0, 6, 4, 26, 24, 30, 28, 18, 16, 22, 20,
    ],
    [
        47, 45, 43, 41, 39, 37, 35, 33, 63, 61, 59, 57, 55, 53, 51, 49, 15, 13, 11, 9, 7, 5, 3, 1,
        31, 29, 27, 25, 23, 21, 19, 17, 44, 46, 40, 42, 36, 38, 32, 34, 60, 62, 56, 58, 52, 54, 48,
        50, 12, 14, 8, 10, 4, 6, 0, 2, 28, 30, 24, 26, 20, 22, 16, 18,
    ],
    [
        45, 47, 41, 43, 37, 39, 33, 35, 61, 63, 57, 59, 53, 55, 49, 51, 13, 15, 9, 11, 5, 7, 1, 3,
        29, 31, 25, 27, 21, 23, 17, 19, 46, 44, 42, 40, 38, 36, 34, 32, 62, 60, 58, 56, 54, 52, 50,
        48, 14, 12, 10, 8, 6, 4, 2, 0, 30, 28, 26, 24, 22, 20, 18, 16,
    ],
    [
        51, 49, 55, 53, 59, 57, 63, 61, 35, 33, 39, 37, 43, 41, 47, 45, 19, 17, 23, 21, 27, 25, 31,
        29, 3, 1, 7, 5, 11, 9, 15, 13, 48, 50, 52, 54, 56, 58, 60, 62, 32, 34, 36, 38, 40, 42, 44,
        46, 16, 18, 20, 22, 24, 26, 28, 30, 0, 2, 4, 6, 8, 10, 12, 14,
    ],
    [
        49, 51, 53, 55, 57, 59, 61, 63, 33, 35, 37, 39, 41, 43, 45, 47, 17, 19, 21, 23, 25, 27, 29,
        31, 1, 3, 5, 7, 9, 11, 13, 15, 50, 48, 54, 52, 58, 56, 62, 60, 34, 32, 38, 36, 42, 40, 46,
        44, 18, 16, 22, 20, 26, 24, 30, 28, 2, 0, 6, 4, 10, 8, 14, 12,
    ],
    [
        55, 53, 51, 49, 63, 61, 59, 57, 39, 37, 35, 33, 47, 45, 43, 41, 23, 21, 19, 17, 31, 29, 27,
        25, 7, 5, 3, 1, 15, 13, 11, 9, 52, 54, 48, 50, 60, 62, 56, 58, 36, 38, 32, 34, 44, 46, 40,
        42, 20, 22, 16, 18, 28, 30, 24, 26, 4, 6, 0, 2, 12, 14, 8, 10,
    ],
    [
        53, 55, 49, 51, 61, 63, 57, 59, 37, 39, 33, 35, 45, 47, 41, 43, 21, 23, 17, 19, 29, 31, 25,
        27, 5, 7, 1, 3, 13, 15, 9, 11, 54, 52, 50, 48, 62, 60, 58, 56, 38, 36, 34, 32, 46, 44, 42,
        40, 22, 20, 18, 16, 30, 28, 26, 24, 6, 4, 2, 0, 14, 12, 10, 8,
    ],
    [
        59, 57, 63, 61, 51, 49, 55, 53, 43, 41, 47, 45, 35, 33, 39, 37, 27, 25, 31, 29, 19, 17, 23,
        21, 11, 9, 15, 13, 3, 1, 7, 5, 56, 58, 60, 62, 48, 50, 52, 54, 40, 42, 44, 46, 32, 34, 36,
        38, 24, 26, 28, 30, 16, 18, 20, 22, 8, 10, 12, 14, 0, 2, 4, 6,
    ],
    [
        57, 59, 61, 63, 49, 51, 53, 55, 41, 43, 45, 47, 33, 35, 37, 39, 25, 27, 29, 31, 17, 19, 21,
        23, 9, 11, 13, 15, 1, 3, 5, 7, 58, 56, 62, 60, 50, 48, 54, 52, 42, 40, 46, 44, 34, 32, 38,
        36, 26, 24, 30, 28, 18, 16, 22, 20, 10, 8, 14, 12, 2, 0, 6, 4,
    ],
    [
        63, 61, 59, 57, 55, 53, 51, 49, 47, 45, 43, 41, 39, 37, 35, 33, 31, 29, 27, 25, 23, 21, 19,
        17, 15, 13, 11, 9, 7, 5, 3, 1, 60, 62, 56, 58, 52, 54, 48, 50, 44, 46, 40, 42, 36, 38, 32,
        34, 28, 30, 24, 26, 20, 22, 16, 18, 12, 14, 8, 10, 4, 6, 0, 2,
    ],
    [
        61, 63, 57, 59, 53, 55, 49, 51, 45, 47, 41, 43, 37, 39, 33, 35, 29, 31, 25, 27, 21, 23, 17,
        19, 13, 15, 9, 11, 5, 7, 1, 3, 62, 60, 58, 56, 54, 52, 50, 48, 46, 44, 42, 40, 38, 36, 34,
        32, 30, 28, 26, 24, 22, 20, 18, 16, 14, 12, 10, 8, 6, 4, 2, 0,
    ],
];

#[cfg(test)]
mod tests {
    /// From wikipedia:
    ///
    /// A quasigroup (Q, ∗) is a non-empty set Q with a binary operation ∗
    /// (that is, a magma, indicating that a quasigroup has to satisfy closure property),
    /// obeying the Latin square property.
    /// This states that, for each a and b in Q,
    /// there exist unique elements x and y in Q such that both
    ///
    /// a * x = b,
    ///
    /// y * a = b
    ///
    /// hold.
    fn is_quasigroup<const N: usize>(f: [[u8; N]; N]) -> bool {
        let mut rhs_buf: [bool; N];
        for a in 0..N {
            // Test a * x = b for all b
            rhs_buf = [false; N];
            for x in 0..N {
                rhs_buf[f[a][x] as usize] = true;
            }
            if !rhs_buf.iter().all(|b| *b) {
                return false;
            }

            // Test y * a = b for all b
            rhs_buf = [false; N];
            for y in 0..N {
                rhs_buf[f[y][a] as usize] = true;
            }
            if !rhs_buf.iter().all(|b| *b) {
                return false;
            }
        }
        true
    }

    /// From wikipedia:
    ///
    /// A quasigroup (Q, ∗) is called weakly totally anti-symmetric if for all
    /// c, x, y ∈ Q, the following implication holds.
    ///
    /// (c ∗ x) ∗ y = (c ∗ y) ∗ x implies that x = y
    fn is_weakly_totally_anti_symmetric<const N: usize>(f: [[u8; N]; N]) -> bool {
        for c in 0..N {
            for x in 0..N {
                for y in 0..N {
                    if (f[f[c][x] as usize][y] == f[f[c][y] as usize][x]) && (x != y) {
                        return false;
                    }
                }
            }
        }
        true
    }

    use super::*;

    #[test]
    fn base4_is_weakly_totally_antisymmetric_quasigroup() {
        assert!(is_quasigroup(BASE4));
        assert!(is_weakly_totally_anti_symmetric(BASE4));
    }

    #[test]
    fn base8_is_weakly_totally_antisymmetric_quasigroup() {
        assert!(is_quasigroup(BASE8));
        assert!(is_weakly_totally_anti_symmetric(BASE8));
    }

    #[test]
    fn base10_is_weakly_totally_antisymmetric_quasigroup() {
        assert!(is_quasigroup(BASE10));
        assert!(is_weakly_totally_anti_symmetric(BASE10));
    }

    #[test]
    fn base12_is_weakly_totally_antisymmetric_quasigroup() {
        assert!(is_quasigroup(BASE12));
        assert!(is_weakly_totally_anti_symmetric(BASE12));
    }

    #[test]
    fn base16_is_weakly_totally_antisymmetric_quasigroup() {
        assert!(is_quasigroup(BASE16));
        assert!(is_weakly_totally_anti_symmetric(BASE16));
    }

    #[test]
    fn base24_is_weakly_totally_antisymmetric_quasigroup() {
        assert!(is_quasigroup(BASE24));
        assert!(is_weakly_totally_anti_symmetric(BASE24));
    }

    #[test]
    fn base26_is_weakly_totally_antisymmetric_quasigroup() {
        assert!(is_quasigroup(BASE26));
        assert!(is_weakly_totally_anti_symmetric(BASE26));
    }

    #[test]
    fn base32_is_weakly_totally_antisymmetric_quasigroup() {
        assert!(is_quasigroup(BASE32));
        assert!(is_weakly_totally_anti_symmetric(BASE32));
    }

    #[test]
    fn base36_is_weakly_totally_antisymmetric_quasigroup() {
        assert!(is_quasigroup(BASE36));
        assert!(is_weakly_totally_anti_symmetric(BASE36));
    }

    #[test]
    fn base40_is_weakly_totally_antisymmetric_quasigroup() {
        assert!(is_quasigroup(BASE40));
        assert!(is_weakly_totally_anti_symmetric(BASE40));
    }

    #[test]
    fn base45_is_weakly_totally_antisymmetric_quasigroup() {
        assert!(is_quasigroup(BASE45));
        assert!(is_weakly_totally_anti_symmetric(BASE45));
    }

    #[test]
    fn base56_is_weakly_totally_antisymmetric_quasigroup() {
        assert!(is_quasigroup(BASE56));
        assert!(is_weakly_totally_anti_symmetric(BASE56));
    }

    #[test]
    fn base58_is_weakly_totally_antisymmetric_quasigroup() {
        assert!(is_quasigroup(BASE58));
        assert!(is_weakly_totally_anti_symmetric(BASE58));
    }

    #[test]
    fn base62_is_weakly_totally_antisymmetric_quasigroup() {
        assert!(is_quasigroup(BASE62));
        assert!(is_weakly_totally_anti_symmetric(BASE62));
    }

    #[test]
    fn base64_is_weakly_totally_antisymmetric_quasigroup() {
        assert!(is_quasigroup(BASE64));
        assert!(is_weakly_totally_anti_symmetric(BASE64));
    }
}
